<?php
############################################################################
//  Generate the [X]HTML for the Manitoba Weather Network links page
//  Based on SWN-mesomap.php for Southwestern Weather Network
//  Version 1.00 - 29-Apr-2008  Author: Ken True, webmaster@saratoga-weather.org
//  Version 1.01 - 10-May-2008 -- added Metric/English units handling
//  Version 1.02 - 23-Aug-2008 -- added sortable table columns and temperatures to 1 decimal place
//  Version 1.03 - 26-Aug-2008 -- added dewpt, wind gust and units conversion for stickertags stations
//  Version 2.00 (ML) - 24-Feb-2009 -- added multingual support via language-meso-LL.txt files for static text
//  Version 2.01 (ML) - 06-Mar-2009 -- added optional rotational display of city name
//  Version 2.02 (ML) - 14-Mar-2009 -- translation for date/time + 'This station'
//  Version 2.03 (ML) - 04-Mar-2010 -- added support for town-name in main local language
//  Version 2.04 (ML) - 14-Jul-2010 -- added support for gzipped conditions data from stations
//
//  Note: distribution of this program is limited to members of the
//  Manitoba Weather Network (http://mb.westerncanadawx.net/ )
//  Distribution outside of the membership is prohibited.
//  Copyright 2006-2009, Ken True - Saratoga-weather.org
//
$Version = "V2.04 (ML) 14-Jul-2010";
//  Inputs:
//    a '|' -delimited data file with the information on the links to generate
//  PHP page parameters:
//   mode=[HTML] | [XHTML]  default=HTML 4.0 format
// ------------- Settings ----------------------------------------
// Note: make changes in CMBWN-meosmap-config.txt instead of here
//
$LinksFile = "CMBWN-stations-cc.txt"; // master control file
//
$Graphic = "CMBWN_meso.jpg";  // used for display on this page and in code gen.
// $Graphic = "./CMBWN_meso.jpg";  // used for display on this page and in code gen.
//
$ThisStation = "This station"; // change to your station name like
//$ThisStation = "Saratoga-weather.org"; // or
//$ThisStation = "Stillweather.com";

//$timeFormat = '%a, %Y-%b-%d %H:%M:%S %Z';  // Fri, 2006-03-31 14:03:22 TZone
//  $timeFormat = '%a, %d-%b-%Y %H:%M:%S %Z';  // Fri, 31-Mar-2006 14:03:22 TZone
  $timeFormat = '%a, %d-%b-%Y %H:%M:%S %Z';  // Fri, 31-Mar-2006 14:03:22 TZone
  
  $ourTZ = "America/Edmonton";  //NOTE: this *MUST* be set correctly to
// translate UTC times to your LOCAL time for the displays.
// cacheName is name of file used to store cached NDBC webpage
// 
  $windArrowDir = './CMBWN-images/'; // set to directory with wind arrows, or
//                        set to '' if wind arrows are not wanted
//                        the program will test to see if images are
//                        available, and set it to '' if no images are
//                        found in the directory specified.
//
//                        // used for rotating legend display :
  $windArrowSize = 'S';   // ='S' for Small 9x9 arrows   (*-sm.gif)
//                           ='L' for Large 14x14 arrows (*.gif)
//
  $showNoData = true;   // show table rows with no conditions data available
//
  $maxAge = 62*60;     // max age in seconds for conditions
//                       (62*60 = 62 minutes)
  $maxAgeMetar = 90*60; // max age for Metar is 90 minutes V2.10
//
//
// cacheName is name of file used to store cached current conditions
//
  $cacheName = "CMBWN-conditions.txt";  // used to store the file so we
//                                        don't have to fetch it each time
//                                        cache is normally filed from master site
//
  $refetchSeconds = 600;     // refetch every nnnn seconds (600=10 minutes)
//
// V2.10 added baro, trend, conditions
  $showBaroTrendArrow = true; // set to false to suppress trend arrows
  $showMapCondIcons = true;   // set to false to supress tiny icons on map

  $includeConfig = true;     // set to true to allow CMBWN-mesomap-config.txt
  	                          // to override the current settings

  $showTownName = true;      // set to false to suppress town name in rotating conditions
  $condIconsDir = './CMBWN-images/'; // for condition icons (same as Anole's)
//
// V2.09 - URL to fetch master cached conditions from
  $masterCacheURL = 'http://mb.westerncanadawx.net/CMBWN-conditions.txt';
//  $masterCacheURL = '';
//   set to '' to disable master cache url fetch.
//
// V2.10 optional -- record raw fetch stats.. set to '' to disable
  $statsDir = ''; // directory to store raw fetch stats.
// 
// V2.14 optional - display this text on map if no station report 
  $NAtext = 'Offline';  // text to display on rotating conditions if no station report
//  
  $myUOM = 'M'; // ='M' metric units, ='E' English(USA) units
// pick ONE of the following to override wind units of kmh for Metric or mph for English units  
  $useKnots = false;  // =true - show wind in Knots only
  $useMPH   = false;  // =true - show wind in Miles-per-hour only
  $useMPS   = false;  // =true - show wind in Meters-per-second only
//
  $useMB    = false;   // for $myUOM = 'M', =true to display 'mb', =false to display 'hPa' as baro units 

// 
  $mainLang = 'en';    // main language for network
// -------------------------------------------------------------
// ------- end of settings ------

/// ------  begin code ----------
if (isset($_REQUEST['sce']) && strtolower($_REQUEST['sce']) == 'view' ) {
   //--self downloader --
   $filenameReal = __FILE__;
   $download_size = filesize($filenameReal);
   header('Pragma: public');
   header('Cache-Control: private');
   header('Cache-Control: no-cache, must-revalidate');
   header("Content-type: text/plain");
   header("Accept-Ranges: bytes");
   header("Content-Length: $download_size");
   header('Connection: close');

   readfile($filenameReal);
   exit;
}

// Map specific settings .. DO NOT CHANGE unless graphic is changed to match these

$MapWidth = 448;   // map image width in px
$MapHeight = 724;  // map image height in px

$LegendX = 6;    // left: for rotating legend display in px
$LegendY = 35;     // top: for rotating legend display in px

$ControlsX = 4;  // left: for control display in px
$ControlsY = 5;  // top: for control display in px

$lang = 'en'; // default language

//------------ end of map-specific settings ---------------


// include the settings override file if it exists
if ($includeConfig and file_exists("CMBWN-mesomap-config.txt")) {
  require_once("CMBWN-mesomap-config.txt");
}

  if (isset($_REQUEST['lang'])) { $lang_input = strtolower($_REQUEST['lang']); }

  # allow valid input only
  # allows en, en-us, en-gb, dk, etc.
  if ((isset($lang_input) && preg_match('/^[a-z]{2}$/', $lang_input))
  || (isset($lang_input) && preg_match('/^[a-z]{2}-[a-z]{2}$/', $lang_input)) ) {
    $lang = $lang_input;
  }

  if(file_exists("CMBWN-language-meso-$lang.txt") ) {
	 $Debug = "<!-- CMBWN-language-meso-$lang.txt loaded. -->\n";
 	 include_once("CMBWN-language-meso-$lang.txt");
	 if ($ThisStation == 'This station') {$ThisStation = MESO_THISSTATION; }
	 if (MESO_OFFLINE <> '') {$NAtext = MESO_OFFLINE; }
  } else {
	$Debug = "<!-- CMBWN-language-meso-$lang.txt not found.  Using CMBWN-language-meso-en.txt -->\n";
	include_once("CMBWN-language-meso-en.txt");
	 if ($ThisStation == 'This station') {$ThisStation = MESO_THISSTATION; }
	 if (MESO_OFFLINE <> '') {$NAtext = MESO_OFFLINE; }
	$lang = 'en';
  }
iconv_set_encoding("output_encoding", MESO_CHARSET);
// Check parameters and force defaults/ranges
  $IncludeMode = false;
  $PrintMode = true;
  $showNoData = false;   // exclude table rows with no conditions data available.. screws up sort if enabled
  
  if (isset($doPrintCMBWN) && ! $doPrintCMBWN ) {
      $PrintMode = false;
  }
  if (isset($_REQUEST['inc']) && 
      strtolower($_REQUEST['inc']) == 'noprint' ) {
	  $PrintMode = false;
  }

if (isset($_REQUEST['inc']) && strtolower($_REQUEST['inc']) == 'y') {
  $IncludeMode = true;
}
if (isset($doIncludeCMBWN)) {
  $IncludeMode = $doIncludeCMBWN;
}


if ( ! isset($_REQUEST['inc']) )
        $_REQUEST['inc']="";
$includeOnly = strtolower($_REQUEST['inc']); // any nonblank is ok
if ($includeOnly) {$includeOnly = "Y";}
if (isset($_REQUEST['inc']) and strtoupper($_REQUEST['inc']) == 'CSS') {$CSSonly = true;} else {$CSSonly = false;}

// show map with hotspots outlined
if (isset($_REQUEST['show']) and strtolower($_REQUEST['show']) == 'map' ) {
 $ShowMap = '&amp;show=map';
 $genJPEG = true;
} else {
 $ShowMap = '';  // no outlines for map
 $genJPEG = false;
}


if($windArrowDir && ! file_exists("$windArrowDir" . 'NNE.gif') ) {
   $windArrowDir = '';  // bad spec.. no arrows found
}

if($condIconsDir && ! file_exists("$condIconsDir" . 'day_clear.gif') ) {
   $condIconsDir = '';  // bad spec.. no icons found
}
 
if (isset($_REQUEST['cfg']) and strtolower($_REQUEST['cfg']) == 'debug') {
  $doDebug = true; 
  } else {
  $doDebug = false;
}
$doDebug = true;

if (isset($_REQUEST['cache']) and strtolower($_REQUEST['cache']) == 'no') {
  $refetchSeconds = 10;  // set short period for refresh of cache
}

$maxAge = $maxAge + $refetchSeconds + 10; // no age penalty from caching
$maxAgeMetar = $maxAgeMetar + $refetchSeconds + 10; // no cache penalty

// HTML specific overrides if desired
$otherParms = '';
if (isset($_REQUEST['gen']) and strtolower($_REQUEST['gen']) == 'xhtml' ) {
 $TARGET = '';  // no target for XHTML 1.0-Strict
 $otherParms = "&amp;gen=xhtml";
} else {
 $TARGET = "target=\"_blank\"";
 $otherParms = "&amp;gen=html";
 
}
if(!isset($PHP_SELF)) {$PHP_SELF = $_SERVER['PHP_SELF']; }

$t = pathinfo($PHP_SELF);
$Program = $t['basename'];
$ourHost = $_SERVER['HTTP_HOST'];
$mc = parse_url($masterCacheURL);
$masterHost = $mc['host'];
$onMasterHost = false;
if ($ourHost == $masterHost) {$onMasterHost = true; }
// print "<!-- ourHost='$ourHost' mH='$masterHost' onM='$onMasterHost' -->\n";

// Establish timezone offset for time display
if (!function_exists('date_default_timezone_set')) {
	
	if (! ini_get('safe_mode') ) {
	   putenv("TZ=$ourTZ");  // set our timezone for 'as of' date on file
	}
//	 print "<!-- using putenv(\"TZ=". $SITE['tz']. "\") -->\n";
  } else {
   date_default_timezone_set($ourTZ);
//	 print "<!-- using date_default_timezone_set(\"". $ourTZ. "\") -->\n";
 }
$locale = setlocale(LC_TIME,$lang.'_'.strtoupper($lang));
$Debug .= "<!-- time locale set to $locale -->\n";
if ($lang == 'el') {
//	error_reporting(E_ALL);
	$locale = setlocale(LC_TIME,'el_EL','ell','gre','grc','greek');
    $Debug .= "<!-- time locale set to $locale -->\n";
}
if ($lang == 'hu') {
//	error_reporting(E_ALL);
	$locale = setlocale(LC_TIME,'hu_HU','hun','hungary','hungarian');
    $Debug .= "<!-- time locale set to $locale -->\n";
}
// ---------------------main program -----------------------------------
// open and read links file
global $Debug,$doDebug,$cacheName,$refetchSeconds,$Icons,$windArrowDir,$wxInfo, $metarPtr, $group, $NAtext, $lang;


$updatedOn = filemtime($LinksFile);  // for our 'as of' date
$updatedOnG = filemtime($Graphic);  // for graphic file

$Stations = array();  // storage area for the station info
$StationData = array(); // storage area for current conditions
$StationStats = array(); // storage for fetch stats for station
$wxInfo = array();
$timeStamp = time();
$currentTime = fixtimezonename(strftime($timeFormat,$timeStamp));

load_config($LinksFile);  // load up the $Stations from the config file

 // show jpg image of map with hotspots outlined
if (isset($_REQUEST['show']) and strtolower($_REQUEST['show']) == 'map' ) {
 $ShowMap = '&amp;show=map';
 $genJPEG = 1;
} else {
 $ShowMap = '';  // no outlines for map
 $genJPEG = 0;
}

  
// show image of map with hotspots outlined
  if (isset($_REQUEST['show']) and strtolower($_REQUEST['show']) == 'hotspots' ) {
    $ourGraphic = $SCRIPT_URI . "?show=map";
    $toggleState = MESO_HOTSGRAP."-- <a href=\"$PHP_SELF?show=normal$otherParms\">".MESO_CLICKNORM ."</a>.\n";
   } else {
    $ourGraphic = $Graphic;  // no outlines for map
    $toggleState = MESO_NORMGRAP." -- <a href=\"$PHP_SELF?show=hotspots$otherParms\">".MESO_HOTSPOTS."</a>.\n";
   }

  if ($genJPEG) { // just produce the map with hotspots outlined
    outline_hotspots($ourGraphic);
	exit;
  }
   	
 
	load_strings();  // Load up the CSS and boilerplates
	
	$Units = load_units($myUOM);
	
	gen_CSS();       // generate the CSS first

    if ($CSSonly) {  // just print the CSS and exit
      print $CSS;
      return;
    }	
	
    load_iconDefs();  // create definitions for condition icons

  load_weather_data();  // this is the time consumer :-)
	

  reset($Stations);  // and reset the list from the config file
  
  $lastState = '';   // to handle the sublists

// generate the links list sorted by State, Station name
  while (list($key, $val) = each($Stations)) { 
    list($State,$Name) = explode("\t",$key);
	list($URL,$Coords,$Features,$DataPage) = explode("\t",$Stations["$key"]);
	
	if ($lastState <> $State) {
	   if ($lastState) {  // not first state
	     $html .= "    </ul>\n";
		 $html .= "  </li>\n";
	   }
       $html .= "  <li>".defregion($State)."\n";
	   $html .= '    <ul style="list-style: circle">' . "\n";
	   $lastState = $State;
	}  // different state
	$t = '';
	if ($TARGET) {$t = " $TARGET";}
	if ($lang == $mainLang and $DataPage <> 'none' and $DataPage <> '') { // substitute display name from stations-cc.txt 
	   $Name = $DataPage;
	}

	$html .= "	      <li><a href=\"$URL\"$t>$Name</a> 
		         [ " . deffeat($Features) . " ]</li>\n";
  }  // end while
  $html .= "    </ul>\n";
  $html .= "  </li>\n";
  $html .= "</ul>\n";

// generate the MAP list sorted by State, Station name
  $html .= '<map name="CMBWN" id="CMBWN">' . "\n";
  
  prt_tablehead(); // initialize the $table area with col heads

  reset($Stations);  // and reset the list so we can start at begining
  
  $lastState = '';   // to handle the sublists
  while (list($key, $val) = each($Stations)) {
    list($State,$Name) = explode("\t",$key);
	list($URL,$Coords,$Features,$DataPage) = explode("\t",$Stations["$key"]);

	prt_tabledata($key);
	if ($lastState <> $State) {
       $html .= "<!-- $State (".defregion($State).") -->\n";
	   $lastState = $State;
	} 
	$t = '';
	if ($TARGET) {$t = "\n                $TARGET";} 
	$html .= "	  <area shape=\"rect\" coords=\"$Coords\" href=\"$URL\" $t
		title=\"" . gen_tag($key) . "\" 
		  alt=\"" . gen_tag($key) . "\" />\n";
  }  // end while

// --------------- customize HTML if you like -----------------------
	     $table .= "</tbody>\n</table>\n<p>&nbsp;</p>\n\n";
// finish up the CSS/HTML assembly strings

  $html .= "</map>
</div> <!-- end html -->
<!-- end of included CMBWN text -->\n";

$oldestData = 9999999999999999;
$newestData = 0;
foreach ($StationData as $key => $vals) {
  list($TEMP,$HUMID,$WDIR,$WSPD,$RAIN,$BARO,$BTRND,$COND,$CTXT,$DEWPT,$GUST,$UDATE,$FTIME) = preg_split("/\,/",$vals);
  if($UDATE > 1000) {
    $oldestData = min($UDATE,$oldestData);
    $newestData = max($UDATE,$newestData);
  }
}
$scroller .= "</div> <!-- end scroller -->\n";
$scroller .= "<p>" . MESO_CONDSFROM. " " . fixtimezonename(strftime($timeFormat,$oldestData)) . " ". MESO_CONDSTO . " " . fixtimezonename(strftime($timeFormat,$newestData)) . " </p>\n"; 

$table .= "<!-- end of included CMBWN table text -->\n";


// text is done now in $html
if ($PrintMode and ! $includeOnly and ! $genJPEG) {  
   // print headers only if inc parm omitted
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=<?php echo MESO_CHARSET; ?>" />
<title>Generate CMBWN mesomap code - <?php echo $Version ?></title>
<style type="text/css">
h1, h2, h3, h4, h5, h6 {
	font-family: Arial,sans-serif;
	margin: 0px;
	padding: 0px;
}

h1{
	font-family: Arial, Helvetica, sans-serif;
	font-size: 120%;
	color: #334d55;
	font-weight: bold;
}

h2{
 font-size: 114%;
 color: #006699;
}
.codebox {
	border: 1px solid #080;
	color: #000000;
	padding: 5px;
	background-color: #FFFF99;
	margin: 5%;
	width: 85%;
}
</style>
<?php 
if ($ShowMap) {
    print preg_replace("|$Graphic|",$ourGraphic,$CSS);
	} else {
	echo $CSS; 
}
?>
</head>
<body style="background-color:#FFFFFF; font-family:Arial, Helvetica, sans-serif; font-size:12px">
<h1><a name="top" id="top"></a><?php echo MESO_LINKGENE ?> - <?php echo $Version; ?></h1>
<?php
} // end if not includeOnly

if ($PrintMode and ! $includeOnly) {  // print headers only if inc parm omitted
$scriptName = substr($PHP_SELF,1);

// Detailed instructions...
print '<p>' . MESO_DETINS1 . '</p>';

// New PHP users...
print '<p>' . MESO_DETINS2 . '</p>';

// PHP users #1 ...
print '<p>' . MESO_DETINS3 . "<a href=\"$Graphic\">" . MESO_DWLD . "</a> " .MESO_UPDATED. "<b> " . fixtimezonename(strftime($timeFormat,$updatedOnG)) . "</b>
<br />&nbsp;&nbsp;&nbsp;". MESO_RIGHTCL . "<b>$Graphic</b>" .MESO_RIGHTSA . '</p>';

// PHP users #2 ...
print '<p>' . MESO_DETINS3 . "<a href=\"$LinksFile\">" . MESO_DWNL . "</a> " . MESO_UPDATED . " <b> "  . fixtimezonename(strftime($timeFormat,$updatedOn)) . "</b>
<br />&nbsp;&nbsp;&nbsp;". MESO_RIGHTCL . "<b>$LinksFile</b>". MESO_RIGHTSA . '</p>';

// PHP users #3 ...
print '<p>' . MESO_DETINS4 . MESO_IFNEED . '</b> <a href="'.$PHP_SELF.'?sce=view">' . MESO_DWNLSC . '</a>' . MESO_FOR . '"' . $scriptName . '" <b>' . $Version . '</b>.
<br />&nbsp;&nbsp;&nbsp;   (' . MESO_ONLYIF . ')
<br />&nbsp;&nbsp;&nbsp;   (' . MESO_SAVEAS . '</p>';

// HTML users...
print "<p>" . MESO_DETINS5 . " <a href=\"http://mb.westerncanadawx.net/CMBWN-code.php#HTMLprocess\">" . MESO_IFRAME . "</p>\n";
// Top of page | Preview of code
print "<p><a href=\"#top\">" . MESO_TOPOF . "</a> | <a href=\"#preview\">". MESO_PREW. "</a></p>";

print "<div class=\"codebox\"><h1><a name=\"preview\"></a>" . MESO_PREB . " CMBWN meso-map</h1>\n<p>$toggleState</p>\n";
print $htmlstart . $ourGraphic . $htmlnext . $scroller . $html;

print prt_jscript();
	print $table;
	if ($doDebug) { print $Debug; }

print '
</div> 
<p><a name="body" id="body"></a>'.MESO_GEN.' ';
if ($TARGET <> '') {
  print "<b>HTML 4.01</b>";
  $toggleGenState = "<a href=\"$PHP_SELF?show=" . $_REQUEST['show'] . "&amp;gen=xhtml\"><b>XHTML 1.0-Strict</b></a>";
  } else {
  print "<b>XHTML 1.0-Strict</b>";
  $toggleGenState = "<a href=\"$PHP_SELF?show=" . $_REQUEST['show'] . "&amp;gen=html\"><b>HTML 4.01</b></a>";
  }
print ' '.MESO_CODECHG. ' '. $toggleGenState . ')</p>
';


$scriptName = substr($PHP_SELF,1);
print "
<p><a href=\"#top\">". MESO_TOPOF . "</a> | <a href=\"#preview\">" . MESO_PREW . "</a> </p>
<p>" . MESO_GEN_ED . " \"$scriptName\" - $Version . <br />"
. MESO_AUTHOR . ": Ken True - <a href=\"mailto:webmaster@saratoga-weather.org\">webmaster@saratoga-weather.org</a></p>
</body>
</html>
";
}  else if ($PrintMode) { // if including .. just print it

    print $htmlstart . $ourGraphic . $htmlnext . $scroller . $html;
	print prt_jscript();
	print $table;
	if ($doDebug) {print $Debug; }
  
} else { // end of if not includeOnly 
global $CMBWN_MAP, $CMBWN_TABLE, $CMBWN_CSS;

$CMBWN_MAP = $htmlstart . $ourGraphic . $htmlnext . $scroller . $html . prt_jscript();
$CMBWN_TABLE = $table;
$CMBWN_CSS = $CSS;
if ($doDebug) {$CMBWN_TABLE .= $Debug; }

}
return;

//-------------------end of main program -------------------


//----------------------------functions ----------------------------------- 

// Generate CSS for positioning rollover conditions display

function gen_CSS () {
  global $CSS, $Stations, $LegendX, $LegendY;
  
  $Top = 35;  // default location for values legend on map
  $Left = 350;
  if ($LegendX) {$Left = $LegendX;}
  if ($LegendY) {$Top = $LegendY;}
 //------------customize this CSS entry for font/color/background for legend 
	$CSS .= "#mesolegend {
      top:  ${Top}px;
      left: ${Left}px;
	  font-family: Arial, Helvetica, sans-serif;
	  font-size: 14pt;
	  font-weight: bold;
	  color: #FFFFFF;
	  padding: 6px 6px;
	  border: 1px solid white;
}
";  
  
  while (list($key, $val) = each($Stations)) { 
    list($State,$Name,$Seqno) = explode("\t",$key);
	list($URL,$Coords,$Features,$DataPage,$RawDataType,$RawDataURL,$Offsets) = explode("\t",$Stations["$key"]);
	$URLparts = parse_url($URL);
	$host = $URLparts['host'];
	$hostalias = preg_replace('/www\.|\.\w{3}|\W/s','',$host);
	$hostalias = preg_replace('/weather/','wx',$hostalias);
	$hostalias = 'C' . $hostalias . "S$Seqno";
	
// generate the CSS to place the rotating display over the map
    $Coords = preg_replace("|\s|is","",$Coords);
	list($Left,$Top,$Right,$Bottom) = explode(",",$Coords . ',,,,');
	list($OLeft,$OTop) = explode(",",$Offsets . ',,');
 
    if (! $Offsets) {  // use default positioning
      $Top = $Top - 2;
      $Left = $Right + 2;
	} else {  // use relative positioning from bottom/left
	  $Top = $Bottom + $OTop;
	  $Left = $Right + $OLeft;
	}

	$CSS .= "#$hostalias {
      top:  ${Top}px;
      left: ${Left}px;
}
";
	

  } // end while $Stations
  $CSS .= "</style>\n<!-- end CMBWN mesomap CSS -->\n";

  reset($Stations);  // and reset the list
  
  return;
} // end gen_CSS
// ------------------------------------------------------------------

//  produce the table heading row 
function prt_tablehead ( ){

global $Units, $table, $scroller, $CSS, $LegendX, $LegendY, $showTownName;
// --------------- customize HTML if you like -----------------------
	    $table .= "
<p class=\"CMBWNtable\"><small>". MESO_COLSORT . "</small></p>
<table border=\"0\" class=\"sortable CMBWNtable\" cellspacing=\"2\">
 <thead>
 <tr>
  <th style=\"text-align: left;vertical-align: top;cursor: n-resize;\"><br />".MESO_STATE."</th>
  <th style=\"text-align: left;vertical-align: top;cursor: n-resize;\"><br />".MESO_STATION."</th>
  <th style=\"text-align: center;vertical-align: top;\" class=\"sorttable_nosort\">".MESO_CURHEAD."</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\">".MESO_TEMP."<br />" . $Units['temp'] . "</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\">".MESO_DEWPT."<br />" . $Units['temp'] . "</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\">".MESO_HUM."<br />" . $Units['humid'] . "</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\" class=\"sorttable_numeric\">".MESO_AVGWIND."<br/>" . $Units['wind'] . "</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\" class=\"sorttable_numeric\">".MESO_GUSTWIND."<br/>" . $Units['wind'] . "</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\">".MESO_PRECIPS."<br />" . $Units['rain'] . "</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\">".MESO_BAROB."<br />" . $Units['baro'] . "</th>
  <th style=\"text-align: center;vertical-align: top;\" class=\"sorttable_nosort\">".MESO_BAROT."</th>
  <th style=\"text-align: center;vertical-align: top;cursor: n-resize;\">".MESO_DATAUPDT."</th>
</tr>
</thead>
<tbody>
";

    $scroller .= "<p id=\"mesolegend\">
  <span class=\"CMBWNcontent0\"><strong>&nbsp;".MESO_TEMPL."&nbsp;</strong></span>
  <span class=\"CMBWNcontent1\"><strong>&nbsp;".MESO_DEWPT."&nbsp;</strong></span>
  <span class=\"CMBWNcontent2\"><strong>&nbsp;".MESO_HUML."&nbsp;</strong></span>
  <span class=\"CMBWNcontent3\"><strong>&nbsp;".MESO_WINDL."&nbsp;</strong></span>
  <span class=\"CMBWNcontent4\"><strong>&nbsp;".MESO_PRECIPSL."&nbsp;</strong></span>
  <span class=\"CMBWNcontent5\"><strong>&nbsp;".MESO_BAROL."&nbsp;</strong></span>
  <span class=\"CMBWNcontent6\"><strong>&nbsp;".MESO_CONDL."&nbsp;</strong></span>";
  if($showTownName) {
	  $scroller .= "  <span class=\"CMBWNcontent7\"><strong>&nbsp;".MESO_TOWN."&nbsp;</strong></span>";
  }
  $scroller .= "</p>\n";

   $scroller .= '<form action="">
<p id="CMBWNcontrols">
<input type="button" value="'.MESO_RUN.'" name="run" onclick="CMBWN_set_run(1);" />
<input type="button" value="'.MESO_PAUSE.'" name="pause" onclick="CMBWN_set_run(0);" />
<input type="button" value="'.MESO_STEP.'" name="step" onclick="CMBWN_step_content();" />
</p>
</form>
';



return;
}  // end function prt_tablehead
// ------------------------------------------------------------------

// produce one row of current conditions data
function prt_tabledata($key) {

 global $StationData,$Stations,$Units,$table,$scroller,$CSS,$StationStats, $Debug;
 global $skipNoData,$windArrowDir,$showNoData,$TARGET, $windArrowSize,$Icons,$condIconsDir;
 global $IconsText, $showBaroTrendArrow, $showMapCondIcons, $NAtext, $showTownName, $mainLang, $lang;

  if ($skipNoData && ! isset($StationData["$key"])) { return; }
  
    list($State,$Name,$Seqno) = explode("\t",$key);
	list($URL,$Coords,$Features,$DataPage,$RawDataType,$RawDataURL) = explode("\t",$Stations["$key"]);
	$URLparts = parse_url($URL);
	$host = $URLparts['host'];
	$hostalias = preg_replace('/www\.|\.\w{3}|\W/s','',$host);
	$hostalias = preg_replace('/weather/','wx',$hostalias);
	$hostalias = 'C' . $hostalias . "S$Seqno";
	
	if ($windArrowSize == 'S') {
	  $windGIF = '-sm.gif';
	  $windSIZE = 'height="9" width="9"';
	} else {
	  $windGIF = '.gif';
	  $windSIZE = 'height="14" width="14"';
	}
//	$Debug .= "<!-- lang=$lang mainLang=$mainLang Name=$Name DataPage=$DataPage -->\n";
    if ($lang == $mainLang and $DataPage <> 'none' and $DataPage <> '') { // substitute display name from stations-cc.txt 
	   $Name = $DataPage;
	}
//    $Debug .= "<!-- Name=$Name -->\n";
 
  if (! isset($StationData["$key"])) {
  
    if ($showNoData) {
      $table .= "
<tr>
  <td>".defregion($State)."</td>
  <td><a href=\"$URL\">$Name</a></td>
  <td colspan=\"10\" align=\"left\">".MESO_NOCOND."</td>
</tr>
 ";
     } // end showNoData
//----------------------------
    $scroller .= "<p id=\"$hostalias\">
  <span class=\"CMBWNcontent0\">$NAtext</span>
  <span class=\"CMBWNcontent1\">$NAtext</span>
  <span class=\"CMBWNcontent2\">$NAtext</span>
  <span class=\"CMBWNcontent3\">$NAtext</span>
  <span class=\"CMBWNcontent4\">$NAtext</span>
  <span class=\"CMBWNcontent5\">$NAtext</span>
  <span class=\"CMBWNcontent6\">$NAtext</span>";
  if($showTownName) {
	  $scroller .= "  <span class=\"CMBWNcontent7\">$Name</span>";
  }
  $scroller .= "</p>\n";

//----------------------------

   return;
   }
// got data for one of our stations.. format the table entry

 	list($TEMP,$HUMID,$WDIR,$WSPD,$RAIN,$BARO,$BTRND,$COND,$CTXT,$DEWPT,$GUST,$UDATE,$FTIME) = preg_split("/\,/",$StationData["$key"]);

// --------------- customize HTML if you like -----------------------
	$t = '';
	if ($TARGET) {$t = " $TARGET";}
	if (preg_match('/Metar/i',$CTXT)) {
		$tCTXT = explode(': ',$CTXT);
		if(isset($cTXT[1])) {$CTXT = $cTXT[1]; }
	}
	$table .= "
<tr>
  <td>".defregion($State)."</td>
  <td><a href=\"$URL\"$t>$Name</a></td>
  <td align=\"center\">";
  if (preg_match('|.gif$|',$COND) && $condIconsDir) {
    $table .= "<img src=\"$condIconsDir" . $COND . "\" height=\"25\" width=\"25\"
	alt=\"".deficons($CTXT)."\" title=\"".deficons($CTXT)."\" />";
  } else {
    $table .= deficons($COND);
  }
  $table .= "</td>
  <td align=\"center\">$TEMP</td>
  <td align=\"center\">$DEWPT</td>
  <td align=\"center\">$HUMID</td>
  <td align=\"right\" style=\"padding-right: 10px;\">"; 
  if ($WDIR == 'n/a') {
    $table .= $WDIR; 
  } else {
    $wda = $WDIR;
	$table .= defdirection($wda);
	if ($windArrowDir) {
       $table .= "&nbsp;<img src=\"$windArrowDir${wda}.gif\" height=\"14\" width=\"14\" 
	    alt=\"".MESO_WINDFROM." ".defdirection($wda) ."\" title=\"".MESO_WINDFROM." ".defdirection($wda) ."\" />";
	}
    $table .=  "&nbsp;" . $WSPD;
  }
  $table .= "</td>
  <td align=\"center\">$GUST</td>
  <td align=\"center\">$RAIN</td>
  <td align=\"center\"><table class=\"CMBWNtable\"><tr><td>$BARO</td><td>";
  if($showBaroTrendArrow) {
    $table .= getBaroTrendArrow($BTRND);
  } 
  $table .="</td></tr></table></td>
  <td align=\"center\">".defbarotrnd($BTRND)."</td>
  <td align=\"center\">" . date('H:i:s',$UDATE) . "</td>
  <!-- $RawDataType load time: $FTIME sec -->
</tr>\n";

// generate the data for the changing conditions display 
// NOTE: changes here may break the rotating conditions display..
    $scroller .= "<p id=\"$hostalias\">
  <span class=\"CMBWNcontent0\">$TEMP " . $Units['temp'] . "</span>
  <span class=\"CMBWNcontent1\">".MESO_DEWPTABBREV." $DEWPT " . $Units['temp'] . "</span>
  <span class=\"CMBWNcontent2\">$HUMID " . $Units['humid'] . "</span>
  <span class=\"CMBWNcontent3\">";
  $wda = '';
  if ($WDIR <> 'n/a') {
    $wda = $WDIR;
	if ($windArrowDir) {
    	$scroller .= "<img src=\"$windArrowDir${wda}${windGIF}\" $windSIZE  
	    alt=\"".MESO_WINDFROM." ".defdirection($wda) ."\" title=\"".MESO_WINDFROM." ".defdirection($wda) ."\" />";
	}
	$gust = '';
	if ($GUST > 0) {$gust = MESO_GUSTABBREV ." $GUST"; }
    $scroller .= defdirection($WDIR) . "&nbsp; $WSPD $gust " . $Units['wind'] ;
  } else {
    $scroller .= defdirection($WDIR);
  }
  $scroller .= "</span>
  <span class=\"CMBWNcontent4\">$RAIN ". $Units['rain'] . "
  </span>
  <span class=\"CMBWNcontent5\">";
  if ($BARO <> '') {
    $scroller .= "$BARO ". $Units['baro'];
	if ($showBaroTrendArrow) {
	  $scroller .= getBaroTrendArrow($BTRND);
	} else {
	  list($t,$q) = explode(" ",$BTRND);
	  $scroller .= " " . defbarotrnd($BTRND);
	}
  } 
  $scroller .= "</span>
  <span class=\"CMBWNcontent6\">";
  if ($showMapCondIcons and preg_match('|.gif$|',$COND) && $condIconsDir) {
      $scroller .= "<img src=\"$condIconsDir" . $COND . "\" height=\"12\" width=\"12\" 
	  alt=\"".deficons($CTXT)."\" title=\"".deficons($CTXT)."\" /> ";
  }
  list($mt,$t) = explode(":",$CTXT . '::');
  if ($t <> '') { 
    $scroller .= deficons(trim($t)); 
	} else {
	$scroller .= deficons(trim($mt));
  }
  $scroller .= "</span>\n";
/*  Fire danger call removed by request
  <span class=\"CMBWNcontent6\">";
  
  $scroller .= getFireDanger($TEMP,$HUMID) ; // Fire danger word+CBI
  
  
  $scroller .= "</span>
  ";
*/
  if($showTownName) {
	  $tname = explode(',',$Name.','); // take the short name if commas are found
	  $tname = $tname[0];
	  $scroller .= "  <span class=\"CMBWNcontent7\">$tname</span>";
  }
  $scroller .= "</p>\n";


return;
} // end prt_tabledata
// ------------------------------------------------------------------

// load JPG image for hotspot work
function loadJPEG ($imgname) { 
   $im = @imagecreatefromjpeg ($imgname); /* Attempt to open */ 
   if (!$im) { /* See if it failed */ 
       $im  = imagecreate (150, 30); /* Create a blank image */ 
       $bgc = imagecolorallocate ($im, 255, 255, 255); 
       $tc  = imagecolorallocate ($im, 0, 0, 0); 
       imagefilledrectangle ($im, 0, 0, 150, 30, $bgc); 
       /* Output an errmsg */ 
       imagestring ($im, 1, 5, 5, "Error loading $imgname", $tc); 
   } 
   return $im; 
} 
// ------------------------------------------------------------------

// load the configuration file and set up $Stations
function load_config($LinksFile) {
   global $Stations;
      $rawlinks = file($LinksFile); // read file into array

	  // strip comment records, build $Stations indexed array
	  $nrec = 0;
	  $Seqno = 0;
      foreach ($rawlinks as $rec) {
	    $Seqno++;
	    $rec = preg_replace("|[\n\r]*|","",$rec);
	    $len = strlen($rec);
	    if($rec and substr($rec,0,1) <> "#") {  //only take non-comments
//	 	   echo "Rec $nrec ($len): $rec\n";
		   list($State,$URL,$Name,$Coords,$Features,$DataPage,$RawDataType,$RawDataURL,$Offsets,$METAR,$LatLong) = explode("|",trim($rec) . '||||||||||||');
		   $Stations["$State\t$Name\t$Seqno"] = "$URL\t$Coords\t$Features\t$DataPage\t$RawDataType\t$RawDataURL\t$Offsets\t$METAR\t$LatLong";  
		   // prepare for sort
//		   echo "<a href=\"$URL\">$Name</a> $State, coord=\"$Coords\"\n";
		   
		} elseif (strlen($rec) > 0) {
//		   echo "comment $nrec ($len): $rec\n";
		} else {
//		   echo "blank record ignored\n";
		}
	    $nrec++;
	  }

  ksort($Stations);  // now sort the keys (state, station name)

}
// ------------------------------------------------------------------

// fetch the weather data from stations and place in $StationData
function load_weather_data() {
  global $Stations,$StationData,$StationRawData,$Debug,$maxAge,
         $cacheName,$refetchSeconds,$masterCacheURL,$onMasterHost,$StationStats,$timeFormat;
		 
// use cache if current and available
if ($cacheName and file_exists($cacheName) and filemtime($cacheName) + $refetchSeconds > time()) {
      $rawcache = file($cacheName);
	  if (preg_match('|\t\d+\||',$rawcache[0])) {
		  $Debug .= "<!-- using Cached version from $cacheName -->\n";
		  foreach ($rawcache as $rec) {
			list($key,$val) = explode('|',trim($rec).'|');
			if ($key <> '') {
			  $StationData["$key"] = $val;
			}
		  }
		  return;
	  } else {
		  $Debug .= "<!-- invalid local cache in $cacheName -->\n";
	  }      
	  
    }
	
// Fetch master cache data if available	
if ($masterCacheURL <> '' and ! $onMasterHost) {
    $Debug .= "<!-- loading from master cache $masterCacheURL -->\n";

    $rawcache = fetchURLWithoutHanging($masterCacheURL,false);
	list($headers,$content) = explode("\r\n\r\n",$rawcache);
	$cache = explode("\n",$content);
	$html = '';
    if ($cache and preg_match('| 200 OK|i',$headers) and preg_match('|\t\d+\||',$cache[0])) {
	  foreach ($cache as $rec) {
	    $html .= trim($rec) . "\n";
	    list($key,$val) = explode('|',trim($rec) . '|');
		if ($key <> '') {
		  $StationData["$key"] = $val;
		}
	  }
      $fp = fopen($cacheName, "w");
	  if (! $fp ) { 
	    $Debug .= "<!-- WARNING: cache $cacheName not writable -->\n";
	  } else {
        $write = fputs($fp, $html);
        fclose($fp);  
        $Debug .= "<!-- wrote local cache to $cacheName. -->\n";
	  }
      return;  
    } else { // failed to get master cache so have to load the hard way
	  $Debug .= "<!-- failed to get master cache -->\n";
	  $Debug .= "<!-- headers: \n " . $headers . " -->\n";
	  $cache = '';
	}
}

  
  $total_time = 0;
  $Debug .= "<!-- fetching data direct from stations -->\n";
  
  while (list($key, $val) = each($Stations)) { 
    list($State,$Name) = explode("\t",$key);
	list($URL,$Coords,$Features,$DataPage,$RawDataType,$RawDataURL,$Offsets,$METAR,$LatLong) = 
	  explode("\t",$Stations["$key"]);
	  
	  
	if (! $RawDataURL && $DataPage) { // fetch datapage instead
	   $RawDataType = 'SWN';
	   $RawDataURL = "$DataPage";
	}
	if(preg_match('|^http|i',$RawDataURL)) {
	  // do nothing.. url fully formed
	 } else {
	  $URLparts = parse_url($URL);
	  $RawDataURL = 'http://' . $URLparts['host'] . '/' . $RawDataURL;
	}
    if($RawDataURL && $RawDataType ) {
	  $Debug .= "\n<!-- type='$RawDataType' RawDataURL = '$RawDataURL' -->\n";
	  $useFopen = false;
	  if ($RawDataType == 'STF' || 
		  $RawDataType == 'CRF') {
	    $useFopen = true;
	  }
	  
      $time_start = microtime_float();

      $rawhtml = fetchUrlWithoutHanging($RawDataURL,$useFopen);

	  $time_stop = microtime_float();
	  $total_time += ($time_stop - $time_start);
	  $time_fetch = sprintf("%01.3f",round($time_stop - $time_start,3));
      $RC = '';
	  if (preg_match("|^HTTP\/\S+ (.*)\r\n|",$rawhtml,$matches)) {
	    $RC = trim($matches[1]);
	  }
	  $Debug .= "<!-- $RawDataType time to fetch: $time_fetch sec ($RC) -->\n";

//  	  list($headers,$content) = explode("\r\n\r\n",$rawhtml);  // split headers from html
      $i = strpos($rawhtml,"\r\n\r\n");
	  $headers = substr($rawhtml,0,$i-1);
	  $content = substr($rawhtml,$i+2);
	  if(preg_match('|Content-Encoding: gzip|Ui',$headers)) {
		  $Debug .= "<!-- Webserver returned gzipped data, dang 'em -- unzipping response -->\n";
		  // $Debug .= "<!-- content='".$content."' -->\n";
		  $err = 'None';
		  $html = explode("\n",trim(my_gzdecode($content,$err)));
		  $Debug .= "<!-- my_gzdecode err='$err' -->\n";
		  if($err <> 'None') {
		    $Debug .= "<!-- contentX='".bin2hex($content).". -->\n";
		  }
	  } else {
	      $html = explode("\n",trim($content));  // put HTML area as separate lines
	  }
      $svrage = 0;
	  $svrtimestamp = time();
	  if(preg_match('|Date: (.*)\n|Ui',$headers,$match)) {
			$svrdate = trim($match[1]);
			$svrtimestamp = strtotime($svrdate);
			$svrage = time() - $svrtimestamp; // server age in seconds
			// do time fixup if need be for hosts with funky Last-modified: dates
		    $Debug .= "<!-- webserver timesync=$svrage sec '$svrdate' -->\n";
	  }
	  $age = 0;

	  if(preg_match('|\nLast-Modified: (.*)\n|Ui',$headers,$match)) {
			$udate = trim($match[1]);
			$utimestamp = strtotime($udate);
			$age = abs($svrtimestamp - $utimestamp); // age in seconds
			// do time fixup if need be for hosts with funky Last-modified: dates
			foreach ($needTimeAdjust as $th => $tadj) {
			  if(preg_match("|".$th."|Ui",$RawDataURL) and 
				$age > 1 * $tadj ) {
				$age = $age - 1 * $tadj;
				$Debug .= "<!-- fixed age for $th (by $tadj seconds) -->\n";
			  }
			}
		    $Debug .= "<!-- age=$age sec '$udate' -->\n";
	  } else {
		  $Debug .= "<!-- server missing Last-Modified: header. Using Date: stamp instead -->\n";
		  $utimestamp = $svrtimestamp;
		  $udate = $svrdate;
		  $age = 0;
	  }

	  
      $metric = implode('',$html);     // now put as one long string
	  $metric = preg_replace('|<[^>]+>|is','',$metric); // clean out any HTML markup
	  $metric = preg_replace('|\s+,|is',',',$metric);     // clean out unwanted spaces
  	  $metric = trim($metric);  // stickertag metric should be ok
	  
	  $Debug .= "<!-- metric='$metric' -->\n";
	  if (! preg_match('|^\d|',$metric)) {
	    $Debug .= "<!-- non-digital data -- html follows \n";
		$tstr = print_r($html,true);
		$tstr = preg_replace('|<|Uis','&lt;',$tstr);
		$tstr = preg_replace('|>|Uis','&gr;',$tstr);
		$Debug .= $tstr;
		$Debug .= "-->\n";
	  } else {
	    if ($age <= $maxAge) {
	      $StationData["$key"] = parse_data($metric,$RawDataType,$METAR) .
		      ",$utimestamp,$time_fetch";  // save away the raw values
	  	  $Debug .= "<!-- $key data='" . $StationData["$key"] . "' -->\n\n";
		} else {
		  $Debug .= "<!-- data age $age more than $maxAge sec .. ignored -->\n\n";
		}
	  }

	} // if $RawDataURL
	$localUdate = fixtimezonename(strftime($timeFormat,strtotime($udate)));
//    $StationStats["$key"] = "$RawDataType\t$time_fetch\t$age\t$localUdate";
  } // end while
  $Debug .= "<!-- total load time: " . round($total_time,3) . " sec -->\n";
  
  if ($cacheName <> '') {
// create cache of values 
    $html = ''; 
    while (list($key, $val) = each($StationData)) { 
      $html .= $key . "|" . $val . "\n";
    }
    $fp = fopen($cacheName, "w");
    if (! $fp ) { 
	    $Debug .= "<!-- WARNING: cache $cacheName not writable -->\n";
	  } else {
        $write = fputs($fp, $html);
        fclose($fp);  
        $Debug .= "<!-- wrote local cache to $cacheName. -->\n";
    }
  }
}
// ------------------------------------------------------------------

function microtime_float()
{
   list($usec, $sec) = explode(" ", microtime());
   return ((float)$usec + (float)$sec);
}

// ------------------------------------------------------------------
function my_gzdecode($rawdata,&$error='',&$filename='',$maxlength=null)
{
	//0d0a1f8b0800000000000003454d390ec3300cdbfb0aee45654a71e463ed5ef40bdd0214feff5adb49130814285122d502535082
	if(strcmp(substr($data,0,2),"\x0d\x0a")) { // got an extra \r\n in the data?
		$data = substr($rawdata,2);
	} else {
		$data = $rawdata;
	}
    $len = strlen($data);
    if ($len < 18 || strcmp(substr($data,0,2),"\x1f\x8b")) {
        $error = "Not in GZIP format.";
        return null;  // Not GZIP format (See RFC 1952)
    }
    $method = ord(substr($data,2,1));  // Compression method
    $flags  = ord(substr($data,3,1));  // Flags
    if ($flags & 31 != $flags) {
        $error = "Reserved bits not allowed.";
        return null;
    }
    // NOTE: $mtime may be negative (PHP integer limitations)
    $mtime = unpack("V", substr($data,4,4));
    $mtime = $mtime[1];
    $xfl   = substr($data,8,1);
    $os    = substr($data,8,1);
    $headerlen = 10;
    $extralen  = 0;
    $extra     = "";
    if ($flags & 4) {
        // 2-byte length prefixed EXTRA data in header
        if ($len - $headerlen - 2 < 8) {
            return false;  // invalid
        }
        $extralen = unpack("v",substr($data,8,2));
        $extralen = $extralen[1];
        if ($len - $headerlen - 2 - $extralen < 8) {
            return false;  // invalid
        }
        $extra = substr($data,10,$extralen);
        $headerlen += 2 + $extralen;
    }
    $filenamelen = 0;
    $filename = "";
    if ($flags & 8) {
        // C-style string
        if ($len - $headerlen - 1 < 8) {
            return false; // invalid
        }
        $filenamelen = strpos(substr($data,$headerlen),chr(0));
        if ($filenamelen === false || $len - $headerlen - $filenamelen - 1 < 8) {
            return false; // invalid
        }
        $filename = substr($data,$headerlen,$filenamelen);
        $headerlen += $filenamelen + 1;
    }
    $commentlen = 0;
    $comment = "";
    if ($flags & 16) {
        // C-style string COMMENT data in header
        if ($len - $headerlen - 1 < 8) {
            return false;    // invalid
        }
        $commentlen = strpos(substr($data,$headerlen),chr(0));
        if ($commentlen === false || $len - $headerlen - $commentlen - 1 < 8) {
            return false;    // Invalid header format
        }
        $comment = substr($data,$headerlen,$commentlen);
        $headerlen += $commentlen + 1;
    }
    $headercrc = "";
    if ($flags & 2) {
        // 2-bytes (lowest order) of CRC32 on header present
        if ($len - $headerlen - 2 < 8) {
            return false;    // invalid
        }
        $calccrc = crc32(substr($data,0,$headerlen)) & 0xffff;
        $headercrc = unpack("v", substr($data,$headerlen,2));
        $headercrc = $headercrc[1];
        if ($headercrc != $calccrc) {
            $error = "Header checksum failed.";
            return false;    // Bad header CRC
        }
        $headerlen += 2;
    }
    // GZIP FOOTER
    $datacrc = unpack("V",substr($data,-8,4));
    $datacrc = sprintf('%u',$datacrc[1] & 0xFFFFFFFF);
    $isize = unpack("V",substr($data,-4));
    $isize = $isize[1];
    // decompression:
    $bodylen = $len-$headerlen-8;
    if ($bodylen < 1) {
        // IMPLEMENTATION BUG!
        return null;
    }
    $body = substr($data,$headerlen,$bodylen);
    $data = "";
    if ($bodylen > 0) {
        switch ($method) {
        case 8:
            // Currently the only supported compression method:
            $data = gzinflate($body,$maxlength);
            break;
        default:
            $error = "Unknown compression method.";
            return false;
        }
    }  // zero-byte body content is allowed
    // Verifiy CRC32
    $crc   = sprintf("%u",crc32($data));
    $crcOK = $crc == $datacrc;
    $lenOK = $isize == strlen($data);
    if (!$lenOK || !$crcOK) {
        $error = ( $lenOK ? '' : 'Length check FAILED. ') . ( $crcOK ? '' : 'Checksum FAILED.');
        return false;
    }
    return $data;
}

// ------------------------------------------------------------------
function parse_data($metric,$RawDataType,$METAR) {
  global $Debug,$Icons,$IconsText,$dpTemp,$dpBaro,$dpRain,$dpWind,$myUOM,$Units;
// from the returned data, select the variables we'll use

  if ($RawDataType == 'ST' || $RawDataType == 'STF') { // a stickertag format
// 0: <!--stationTime-->,
// 1: <!--stationDate-->,
// 2: <!--outsideTemp-->,
// 3: <!--outsideHeatIndex-->,
// 4: <!--windChill-->,
// 5: <!--outsideHumidity-->,
// 6: <!--outsideDewPt-->,
// 7: <!--barometer-->,
// 8: <!--BarTrend-->,
// 9: <!--windSpeed-->,
//10: <!--windDirection-->,
//11: <!--dailyRain-->  // old record ended here '12:44pm,7/3/06,99,101,97,31,63,29.90,Falling,2,SE,0.00,3'
//12: ^curr_cconds1^ in VWS, null for WeatherLink
//13: <!--sunriseTime-->
//14: <!--sunsetTime-->
//15: <!--windAvg10-->,
//16: <!--windHigh10-->,
//17: <!--tempUnit-->|<!--windUnit-->|<!--barUnit-->|<!--rainUnit-->
// '3:55,28/08/08,14.8,14.9,14.8,90,13.2,1016.9,Falling Slowly,1.6,WNW,0.0,, 6:31,20:25,1.8,8.0,C|km/hr|mb|mm'
//
    $v = explode(",",$metric);
    $SuomTemp = $Units['temp'];  // make (possibly bad) assumption that station is reporting using the
    $SuomWind = $Units['wind'];  // same units of measure as the mesomap, so no conversion will be done
    $SuomBaro = $Units['baro'];
    $SuomRain = $Units['rain'];
	if(isset($v[17]) and $v[17] <> '') {
	  $UNITS = explode('|',trim($v[17]).'||||');
	  if($UNITS[3] <> '' and strlen($UNITS[3]) > 2) { $UNITS[3] = substr($UNITS[3],0,2); }
	  if($UNITS[0] <> '') {$SuomTemp = $UNITS[0];}
	  if($UNITS[1] <> '') {$SuomWind = $UNITS[1];}
	  if($UNITS[2] <> '') {$SuomBaro = $UNITS[2];}
	  if($UNITS[3] <> '') {$SuomRain = $UNITS[3];}
	  
	  $Debug .= "<!-- station reporting units $SuomTemp,$SuomWind,$SuomBaro,$SuomRain -->\n";
	} else {
	  $SuomTemp = $Units['temp'];  // make (possibly bad) assumption that station is reporting using the
	  $SuomWind = $Units['wind'];  // same units of measure as the mesomap, so no conversion will be done
	  $SuomBaro = $Units['baro'];
	  $SuomRain = $Units['rain'];
	  $Debug .= "<!-- station using mesomap units $SuomTemp,$SuomWind,$SuomBaro,$SuomRain by default -->\n";
	}
	$TEMP = convertTemp(trim($v[2]),$SuomTemp);
	$HUMID= round(trim($v[5]),0);
	$DEWPT = convertTemp(trim($v[6]),$SuomTemp);
	$WDIR = split(" ",trim($v[10]));
	$WDIR = $WDIR[0]; // handle spaces in wind-dir from edhweather

	if (preg_match('|\d+|',$WDIR)) {
	   $Debug .= "<!-- fixed up WDIR='$WDIR' to '";
	   $WDIR = getWindDir($WDIR);
	   $Debug .= "$WDIR' --?\n";
	 }
	 // NOTE: need units conversion call here......
	$WSPD = convertWind(trim($v[9]),$SuomWind);
	$t = trim($v[15]);
	if (is_numeric($t)) { $WSPD = convertWind($t,$SuomWind); } // use the windAvg10 summary instead for wind speed
	$GUST = '';
	$t = trim($v[16]);
	if (is_numeric($t) and $t > 0) {$GUST = convertWind($t,$SuomWind); } // add  gust to display
	$tRAIN = explode(" ",trim($v[11])); // remove trailing junk from rain value 
	$RAIN = convertRain($tRAIN[0],$SuomRain);
	$BARO = convertBaro(trim($v[7]),$SuomBaro);
	$BTRND = trim($v[8]);
    if (isset($v[14]) and trim($v[12]) <> '') { // decode regular stickertag
	  // VWS Sticker Tag with conditions
	  $COND = decode_VWS_conds(trim($v[0]),trim(strtoupper($v[12])),trim($v[13]),trim($v[14]));
	  $CTXT = trim($v[12]);
    } else {
	  $COND = '';
	  $CTXT = '';
	}
	
    if ($COND == '' && $METAR <> '') {
     list($Sky,$Weather) = explode("|",getMetar($METAR));
	 $COND = decode_VWS_conds(trim($v[0]),strtoupper($Sky),trim($v[13]),trim($v[14]));
	 if ($COND <> '') {
 	 	$CTXT = "Metar $METAR: " . deficons($Sky);
	 }
    }	
  } // end ST type
  
 if ($RawDataType == 'STCU') { // Cumulus

  $v = explode(" ",$metric);
  $SuomTemp = $v[14];
  $SuomWind = $v[13];
  $SuomBaro = $v[15];
  $SuomRain = $v[16];
  $Debug .= "<!-- station reporting units $SuomTemp,$SuomWind,$SuomBaro,$SuomRain -->\n";

   $TEMP = convertTemp($v[2],$SuomTemp);
   $MINITEMP = convertTemp($v[28],$SuomTemp);
   $DEWPT = convertTemp($v[4],$SuomTemp);

   $HUMID= round(trim($v[3]), 0);
   $WDIR = trim($v[11]);

   $WSPD = convertWind( trim($v[5]),$SuomWind);
   $WGST = convertWind( trim($v[40]),$SuomWind);
   $MAXWIND = convertWind($v[30],$SuomWind);
   $MAXGUST = convertWind($v[32],$SuomWind);



   $RAIN = convertRain(trim($v[9]),$SuomRain);
   $YMRAIN = convertRain(trim($v[21]),$SuomRain);
   $MRAIN = convertRain(trim($v[20]),$SuomRain);
   $RRATE = convertRain(trim($v[8]),$SuomRain);


   $BARO = convertBaro(trim($v[10]),$SuomBaro);

   $BTRND = set_barotrend(trim($v[18]));

   $TIME = $v[1];
   $COND = ''; // no conditions from Cumulus realtime.txt file
   if ($METAR <> '') {
     list($Sky,$Weather) = explode("|",getMetar($METAR));
	 $COND = decode_VWS_conds('',strtoupper($Sky),'','');
	 if ($COND <> '') {
	   $CTXT = "Metar $METAR: $Sky";
	 }
   }   

   $OLDRAIN = 0;
  } // end STCU type

  if ($RawDataType == 'CR' || $RawDataType == 'CRF') { 
  // WeatherDisplay clientraw.txt format
  // clientraw is in metric always
  $v = explode(" ",$metric); // clientraw is space-delimited
	  $SuomTemp = 'C';  // clientraw.txt is always in C,kts,hPa,mm
	  $SuomWind = 'kts';  
	  $SuomBaro = 'hPa';
	  $SuomRain = 'mm';
	  $Debug .= "<!-- station using CR units $SuomTemp,$SuomWind,$SuomBaro,$SuomRain by default -->\n";

//    $DATE = "$v[74]";
//    $TIME = sprintf ("%02d:%02d", $v[29], $v[30]);
    $WDIR = getWindDir($v[3]);
    $WSPD = convertWind($v[1],$SuomWind);
    $GUST = convertWind($v[2],$SuomWind); 
    $TEMP = convertTemp($v[4],$SuomTemp);
    $HUMID = round(trim($v[5]),0);
    $BARO = convertBaro($v[6],$SuomBaro);
    $BTRND = set_barotrend($v[50]);
    $RAIN = convertRain($v[7],$SuomRain);
	if(M_NETID == 'NZWN' and isset($v[162]) and !preg_match($v[162],'/^!!C/')) { // NZ Rain value
	  $Debug .= "<!-- using NZ rain cr[162]=". $v[162] . " mm instead of cr[7]=" . $v[7] . " mm -->\n";
	  $RAIN = convertRain($v[162],$SuomRain);
	}

//    $WDCH = round((1.8*$v[44]) + 32),1);
    $DEWPT = convertTemp($v[72],$SuomTemp);
//    $HEAT = round((1.8*$v[112]) + 32),1);
//    $SPCT = "$v[34]";
//    $SOLR = "$v[127]";
//    $UV = "$v[79]";
    $COND = $Icons[$v[48]]; // condition icon
	$CTXT = $IconsText[$v[48]]; // Text description
   } // end CR (clientraw) type
   
   
//V2.01 - bad sensor removal based on SwnData entries
   if(preg_match('/notemp|nohum|norain|nobaro|nowind/i',$DataPage)) {
	   $Debug .= "<!-- Note: station sensor(s) marked offline in ".M_NETID."-stations-cc.txt by '$DataPage' -->\n";
   }
   if(preg_match('/notemp/i',$DataPage)) {
	   $Debug .= "<!-- temp sensor (showing '$TEMP'} marked offline -->\n";
	   $TEMP = '-';
   }
   if(preg_match('/nohum/i',$DataPage)) {
	   $Debug .= "<!-- humidity sensor (showing '$HUMID') marked offline -->\n";
	   $HUMID = '-';
   }
   if(preg_match('/notemp|nohum/i',$DataPage)) {
	   $Debug .= "<!-- dewpoint (showing '$DEWPT') not avalible due to offline sensor(s) -->\n";
	   $DEWPT = '-';
   }
   if(preg_match('/norain/i',$DataPage)) {
	   $Debug .= "<!-- rain sensor (showing '$RAIN') marked offline -->\n";
	   $RAIN = '-';
   }
   if(preg_match('/nobaro/i',$DataPage)) {
	   $Debug .= "<!-- pressure(baro) sensor (showing '$BARO' with trend '$BTRND') marked offline -->\n";
	   $BARO = '-';
	   $BTRND = '-';
   }
    if(preg_match('/nowind/i',$DataPage)) {
	   $Debug .= "<!-- wind sensor (showing  speed '$WSPD' dir '$WDIR' gust '$GUST') marked offline -->\n";
	   $WSPD = '-';
	   $WDIR = '-';
	   $GUST = '-';
   }
 return("$TEMP,$HUMID,$WDIR,$WSPD,$RAIN,$BARO,$BTRND,$COND,$CTXT,$DEWPT,$GUST");
}

// ------------------------------------------------------------
// utility functions to handle conversions from clientraw data to desired units-of-measure
function convertTemp ( $intemp , $tempUOM = 'C') {
   global $myUOM,$dpTemp,$Debug;
   $rawtemp = $intemp;
   if(!preg_match('|C|',$tempUOM)) { // convert to C first
     $rawtemp = (float)(($intemp - 32.0) / 1.8 );
     $Debug .= "<!-- convertTemp($intemp,$tempUOM) to C $rawtemp (unrounded) -->\n";
   }
   
	if ($myUOM == 'E') { // convert C to F
		return( round((1.8 * $rawtemp) + 32.0,$dpTemp));
	} else {  // leave as C
		return (round($rawtemp * 1.0,$dpTemp));
	}
}

function convertWind  ( $inwind, $inUOM = 'KTS' ) {
   global $myUOM,$useKnots,$useMPH,$useMPS,$dpWind,$Debug;
  
   $using = '';
   $WIND = '';
   $cvt = '';
   $rawwind = $inwind;
   if($inUOM <> 'KTS') { // first convert to knots
      if(preg_match('!km!i',$inUOM)) {
	     $rawwind = $rawwind * 0.539956803;  // kmh -> knots
		 $cvt = ',kmh';
	  }
	  if(preg_match('!mph!i',$inUOM)) {
	     $rawwind = $rawwind * 0.868976242;  // mph -> knots
		 $cvt = ',mph';
	  }
	  if(preg_match('!m/s|mps!i',$inUOM)) {
	     $rawwind = $rawwind * 1.94384449;  // m/s -> knots
		 $cvt = ',m/s';
	  }
   
   }
   
	if (($myUOM == 'E' || $useMPH) and (! $useKnots and ! $useMPS)) { // convert knots to mph
		$WIND = sprintf($dpWind?"%02.${dpWind}f":"%d",round($rawwind * 1.1507794,$dpWind));
		$using = 'MPH';
	}   
	if ($useKnots) { $WIND = round($rawwind * 1.0,$dpWind) ;
	  $using='KTS';
	} //force usage of knots for speed
    if ($myUOM == 'M' and $useMPS and ! $useKnots and ! $useMPH ) { // convert knots to m/s
		  $WIND = sprintf($dpWind?"%02.${dpWind}f":"%d",round($rawwind * 0.514444444,$dpWind));
		  $using = 'MPS';
	} 
	if ($myUOM == 'M' and ! $useMPS and ! $useKnots and ! $useMPH) { // convert knots to km/hr
		  $WIND = sprintf($dpWind?"%02.${dpWind}f":"%d",round($rawwind * 1.852,$dpWind));
		  $using = 'KMH';
	}
	$Debug .= "<!-- convertWind($inwind$cvt) using $inUOM to $using is '$WIND' -->\n";
	return($WIND);
}

function convertBaro ( $inpress, $baroUOM = 'hPa' ) {
  global $myUOM,$dpBaro,$Debug;
   $rawpress = $inpress;
   if(preg_match('|in|i',$baroUOM)) { // convert to hPa first
     $rawpress = (float)($inpress * 33.86388158 );  // inHg -> hPa(or mb)
     $Debug .= "<!-- convertBaro($inpress,$baroUOM) to hPa $rawpress (unrounded) -->\n";
   }

	if ($myUOM == 'E') { // convert hPa to inHg
	   return (sprintf("%02.${dpBaro}f",round($rawpress  / 33.86388158,$dpBaro)));
	} else {
	   return (sprintf("%02.${dpBaro}f",round($rawpress * 1.0,$dpBaro))); // leave in hPa
	}
}

function convertRain ( $inrain, $rainUOM = 'mm' ) {
  global $myUOM,$dpRain,$Debug;
  $rawrain = $inrain;
   if(!preg_match('|mm|i',$rainUOM)) { // convert to mm first
     $rawrain = (float)($inrain * 25.4);
     $Debug .= "<!-- convertRain($inrain,$rainUOM) to mm $rawrain (unrounded) -->\n";
   }
	if ($myUOM == 'E') { // convert mm to inches
	   return (sprintf("%02.${dpRain}f",round($rawrain * .0393700787,$dpRain)));
	} else {
	   return (sprintf("%02.${dpRain}f",round($rawrain * 1.0,$dpRain))); // leave in mm
	}
}
// ------------------------------------------------------------

function getMetar($icao) {
  global $Debug,$wxInfo,$maxAgeMetar;
  $Sky = '';
  $Weather = '';
  
  $Debug .= "<!-- METAR='$icao' -->\n";
  $host = 'weather.noaa.gov';
  $path = '/pub/data/observations/metar/stations/';
  $MetarURL = 'http://' . $host . $path . $icao . '.TXT';
 
  $time_start = microtime_float();

  $rawhtml = fetchUrlWithoutHanging($MetarURL,false);
  list($headers,$html) = explode("\r\n\r\n",$rawhtml);
  
  $time_stop = microtime_float();
  $total_time += ($time_stop - $time_start);
  $time_fetch = sprintf("%01.3f",round($time_stop - $time_start,3));
  $Debug .= "<!-- $METAR time to fetch: $time_fetch sec -->\n";
  
  $raw_metar = ereg_replace("[\n\r ]+", ' ', trim(implode(' ', (array)$html)));
  $Debug .= "<!-- '$raw_metar' -->\n";
  $metar = trim($raw_metar);
  $metarDate = preg_replace('|/|','-',substr($metar,0,16)) . ':00 GMT';
  $age = abs(time() - strtotime($metarDate)); // age in seconds
  
  $Debug .= "<!-- age=$age sec '$metarDate' -->\n";
  if ($age >= $maxAgeMetar) {
    $Debug .= "<!-- metar too old.. ignored -->\n";
	return('' . '|' . '');
  }
  error_reporting(E_ALL);
  process_metar($metar,$icao);
  error_reporting(0);
  $Debug .= "<!-- wxInfo \n" . print_r($wxInfo,true) . " -->\n";
  $Sky = trim($wxInfo['CLOUDS']);
  $Weather = trim($wxInfo['CONDITIONS']);

  if ($Sky && $Weather) {
    $Sky = $Weather;
  }
  if ($Sky && ! $Weather) {
    $Weather = $Sky;
  }

  return($Sky . '|' . $Weather);
}  
 // end getMetar functions
// ------------------------------------------------------------

// get contents from one URL and return as string 
 function fetchUrlWithoutHanging($url,$useFopen) {
// thanks to Tom at Carterlake.org for this script fragment
  global $Debug, $needCookie,$timeStamp,$TOTALtime;
  $overall_start = time();
  if (! $useFopen) {
   // Set maximum number of seconds (can have floating-point) to wait for feed before displaying page without feed
   $numberOfSeconds=2;   

   // Suppress error reporting so Web site visitors are unaware if the feed fails
   error_reporting(0);

   // Extract resource path and domain from URL ready for fsockopen
   $FullUrl = $url;
   $urlParts = parse_url($url);
   
   $domain = $urlParts['host'];
   if(isset($urlParts['port'])) {
      $port   = $urlParts['port'];
   } else { 
      $port   = 80;
   }
   $resourcePath = $urlParts['path'];
   $resourcePath = preg_replace('|nocache|','?'.$timeStamp,$resourcePath);
   if(isset($urlParts['query']))    {$resourcePath .= "?" . $urlParts['query']; }
   if(isset($urlParts['fragment'])) {$resourcePath .= "#" . $urlParts['fragment']; }
   $T_start = microtime_float();
   $hostIP = gethostbyname($domain);
   $T_dns = microtime_float();
   $ms_dns  = sprintf("%01.3f",round($T_dns - $T_start,3));
   
   $Debug .= "<!-- GET $resourcePath HTTP/1.1 \n      Host: $domain  Port: $port IP=$hostIP-->\n";
//   print "GET $resourcePath HTTP/1.1 \n      Host: $domain  Port: $port IP=$hostIP\n";

   // Establish a connection
   $socketConnection = fsockopen($hostIP, $port, $errno, $errstr, $numberOfSeconds);
   $T_connect = microtime_float();
   $T_puts = 0;
   $T_gets = 0;
   $T_close = 0;
   
   if (!$socketConnection)
       {
       // You may wish to remove the following debugging line on a live Web site
       $Debug .= "<!-- Network error: $errstr ($errno) -->\n";
//       print "Network error: $errstr ($errno)\n";
       }    // end if
   else    {
       $xml = '';
	   $getString = "GET $resourcePath HTTP/1.1\r\nHost: $domain\r\nConnection: Close\r\n\r\n";
//	   if (isset($needCookie[$domain])) {
//	     $getString .= $needCookie[$domain] . "\r\n";
//		 print " used '" . $needCookie[$domain] . "' for GET \n";
//	   }
	   
//	   $getString .= "\r\n";
//	   print "Sending:\n$getString\n\n";
       fputs($socketConnection, $getString);
       $T_puts = microtime_float();
	   
       // Loop until end of file
	   $TGETstats = array();
	   $TGETcount = 0;
       while (!feof($socketConnection))
           {
		   $T_getstart = microtime_float();
           $xml .= fgets($socketConnection, 8192);
		   $T_getend = microtime_float();
		   $TGETcount++;
		   $TGETstats[$TGETcount] = sprintf("%01.3f",round($T_getend - $T_getstart,3));
           }    // end while
       $T_gets = microtime_float();
       fclose ($socketConnection);
       $T_close = microtime_float();
       }    // end else
   $ms_connect = sprintf("%01.3f",round($T_connect - $T_dns,3));

   if($T_close > 0) {
      $ms_puts = sprintf("%01.3f",round($T_puts - $T_connect,3));
	  $ms_gets = sprintf("%01.3f",round($T_gets - $T_puts,3));
	  $ms_close = sprintf("%01.3f",round($T_close - $T_gets,3));
	  $ms_total = sprintf("%01.3f",round($T_close - $T_start,3)); 
    } else {
       $ms_puts = 'n/a';
	  $ms_gets = 'n/a';
	  $ms_close = 'n/a';
	  $ms_total = sprintf("%01.3f",round($T_connect - $T_start,3)); 
   }

   $Debug .= "<!-- HTTP stats:  dns=$ms_dns conn=$ms_connect put=$ms_puts get($TGETcount blocks)=$ms_gets close=$ms_close total=$ms_total secs -->\n";
//   print  "HTTP stats: dns=$ms_dns conn=$ms_connect put=$ms_puts get($TGETcount blocks)=$ms_gets close=$ms_close total=$ms_total secs \n";
//   foreach ($TGETstats as $block => $mstimes) {
//     print "HTTP Block $block took $mstimes\n";
//   }
   $TOTALtime+= ($T_close - $T_start);
   $overall_end = time();
   $overall_elapsed =   $overall_end - $overall_start;
   $Debug .= "<!-- fetch function elapsed= $overall_elapsed secs. -->\n"; 
//   print "fetch function elapsed= $overall_elapsed secs.\n"; 
   return($xml);

 } else {
//   print "<!-- using file function -->\n";
   $T_start = microtime_float();

   $xml = implode('',file($url));
   $T_close = microtime_float();
   $ms_total = sprintf("%01.3f",round($T_close - $T_start,3)); 
   $Debug .= "<!-- file() stats: total=$ms_total secs -->\n";
//   print " file() stats: total=$ms_total secs.\n";
   $TOTALtime+= ($T_close - $T_start);
   $overall_end = time();
   $overall_elapsed =   $overall_end - $overall_start;
   $Debug .= "<!-- fetch function elapsed= $overall_elapsed secs. -->\n"; 
//   print "fetch function elapsed= $overall_elapsed secs.\n"; 
   return($xml);
 }

   }    // end fetchUrlWithoutHanging
// ------------------------------------------------------------------

// load the image and draw hotspots on it
function outline_hotspots($Graphic) {

    global $Stations;
	
    $image = loadJPEG($Graphic);  // fetch our map image
    $color = imagecolorallocate($image, 0, 255, 0);
	$MaxX = imagesx($image);
	$MaxY = imagesy($image);

    while (list($key, $val) = each($Stations)) { //write each hotspot
      list($State,$Name) = explode("\t",$key);
	  list($URL,$Coords,$Features) = explode("\t",$Stations["$key"]);
      list($X1,$Y1,$X2,$Y2) = explode(",",$Coords);
  	  // make sure images in top-left, bottom-right order
	  if($X1 > $X2) { $tmp = $X2; $X2 = $X1; $X1 = $tmp; }
      if($Y1 > $Y2) { $tmp = $Y2; $Y2 = $Y1; $Y1 = $tmp; }
	  imagerectangle($image, $X1, $Y1, $X2, $Y2, $color);
  
    }
	$msg = MESO_GRAPHICNOSAVE;
    imagestring($image, 3, 45, 2, $msg, $color); 

	header("Content-type: image/jpeg"); // now send to browser
    imagejpeg($image); 
    imagedestroy($image); 

}
// ------------------------------------------------------------------

function getFireDanger( $ctemp, $rh) {
// from SLOWeather.com = calculate fire danger based on temperature and relative humidity

// Convert F temp to C temp
//$ctemp = ($ftemp - 32) * 0.5556;

// Start Index Calcs

// Chandler Index
$cbi = (((110 - 1.373 * $rh) - 0.54 * (10.20 - $ctemp)) * (124 * pow(10,(-0.0142*"$rh"))))/60;
// CBI = (((110 - 1.373*RH) - 0.54 * (10.20 - T)) * (124 * 10**(-0.0142*RH)))/60

//Sort out the Chandler Index

$cbitextcolor = "white";

if ($cbi > "97.5") {
    $cbitxt = "Extreme";
	$cbicolor = "red";
    $cbiimg= "fdl_extreme.gif";

} elseif ($cbi >="90") {
    $cbitxt = "Very High";
	$cbicolor = "orange";
	$cbitextcolor = "black";
    $cbiimg= "fdl_vhigh.gif";

} elseif ($cbi >= "75") {
    $cbitxt = "High";
	$cbicolor = "yellow";
	$cbitextcolor = "black";
    $cbiimg= "fdl_high.gif";

} elseif ($cbi >= "50") {
    $cbitxt = "Moderate";
	$cbicolor = "blue";
    $cbiimg= "fdl_moderate.gif";

} else  {
    $cbitxt="Low";
	$cbicolor = "green";
    $cbiimg= "fdl_low.gif";

}


$val = '<span style="background-color:' . $cbicolor . 
     '; color:' . $cbitextcolor . ';">&nbsp;' .
     $cbitxt . '&nbsp;</span>';
return("$val"); // (CBI " . round($cbi,0) . ")");

}

// ------------------------------------------------------------------

//  convert degrees into wind direction abbreviation   
function getWindDir ($degrees) {
   // figure out a text value for compass direction
   $winddir = $degrees;
   if ($winddir == "n/a") { return($winddir); }
   
   switch (TRUE) {
     case (($winddir >= 349) and ($winddir <= 360)):
       $winddir = 'N';
     break;
     case (($winddir >= 0) and ($winddir <= 11)):
       $winddir = 'N';
     break;
     case (($winddir > 11) and ($winddir <= 34)):
       $winddir = 'NNE';
     break;
     case (($winddir > 34) and ($winddir <= 56)):
       $winddir = 'NE';
     break;
     case (($winddir > 56) and ($winddir <= 78)):
       $winddir = 'ENE';
     break;
     case (($winddir > 78) and ($winddir <= 101)):
       $winddir = 'E';
     break;
     case (($winddir > 101) and ($winddir <= 124)):
       $winddir = 'ESE';
     break;
     case (($winddir > 124) and ($winddir <= 146)):
       $winddir = 'SE';
     break;
     case (($winddir > 146) and ($winddir <= 169)):
       $winddir = 'SSE';
     break;
     case (($winddir > 169) and ($winddir <= 191)):
       $winddir = 'S';
     break;
     case (($winddir > 191) and ($winddir <= 214)):
       $winddir = 'SSW';
     break;
     case (($winddir > 214) and ($winddir <= 236)):
       $winddir = 'SW';
     break;
     case (($winddir > 236) and ($winddir <= 259)):
       $winddir = 'WSW';
     break;
     case (($winddir > 259) and ($winddir <= 281)):
       $winddir = 'W';
     break;
     case (($winddir > 281) and ($winddir <= 304)):
       $winddir = 'WNW';
     break;
     case (($winddir > 304) and ($winddir <= 326)):
       $winddir = 'NW';
     break;
     case (($winddir > 326) and ($winddir <= 349)):
       $winddir = 'NNW';
     break;
   } // end switch

//  return("$winddir ($degrees&deg; true)");
  return("$winddir");

} // end function getWindDir
// ------------------------------------------------------------------


function set_barotrend($baromtrend) {
   // routine from Anole's wxsticker PHP (adapted)
   
//   Barometric Trend(3 hour)

// Change Rates
// Rapidly: =.06" H; 1.5 mm Hg; 2 hPa; 2 mb
// Slowly: =.02" H; 0.5 mm Hg; 0.7 hPa; 0.7 mb

// 5 Arrow Positions:
// Rising Rapidly
// Rising Slowly
// Steady
// Falling Slowly
// Falling Rapidly

// Page 52 of the PDF Manual
// http://www.davisnet.com/product_documents/weather/manuals/07395.234-VP2_Manual.pdf


   // figure out a text value for barometric pressure trend
   settype($baromtrend, "float");
   switch (TRUE) {
      case (($baromtrend >= -.6) and ($baromtrend <= .6)):
        $baromtrendwords = "Steady";
      break;
      case (($baromtrend > .6) and ($baromtrend < 2)):
        $baromtrendwords = "Rising Slowly";
      break;
      case ($baromtrend >= 2):
        $baromtrendwords = "Rising Rapidly";
      break;
      case (($baromtrend < -.6) and ($baromtrend > -2)):
        $baromtrendwords = "Falling Slowly";
      break;
      case ($baromtrend <= -2):
        $baromtrendwords = "Falling Rapidly";
      break;
   } // end switch
  return($baromtrendwords);
}


// generate the alt=/title= text for area statement tooltip popups

function gen_tag($ID) {
   global $StationData,$Stations,$Units,$lang,$mainLang;
   list($State,$Name) = preg_split("/\t/","$ID");
   list($URL,$Coords,$Features,$DataPage,$RawDataType,$RawDataURL) = preg_split("/\t/",$Stations["$ID"]);
   if ($lang == $mainLang and $DataPage <> 'none' and $DataPage <> '') { // substitute display name from stations-cc.txt 
	   $Name = $DataPage;
   }

   if (! isset($StationData["$ID"]) ) {

   return "$Name - " . MESO_NOCOND;
   }

	list($TEMP,$HUMID,$WDIR,$WSPD,$RAIN,$BARO,$BTRND,$COND,$CTXT,$DEWPT,$GUST,$UDATE) = preg_split("/\,/",$StationData["$ID"]);

// --------------- customize HTML if you like -----------------------
// note: only IE supports using new-line and tabs in tooltip displays
//   Firefox, Netscape and Opera all ignore these formatting characters,
//   or display a funky character instead.

$gust = '';
if ($GUST > 0) { $gust = MESO_GUSTABBREV . " $GUST "; }
$tag = "$Name: " .
  "$TEMP". $Units['temp'] . ", " .
  MESO_DEWPTABBREV . " $DEWPT".$Units['temp'] . ", " .
  "$HUMID" . $Units['humid'] . ", " .
  defdirection($WDIR) . " $WSPD $gust" . $Units['wind'] . ", " .
  MESO_PRECIPS .": $RAIN". $Units['rain']  . ", " .
  MESO_BAROB . ": $BARO". $Units['baro'] . " " .
  defbarotrnd($BTRND) . ", ";
  list($mt,$t) = explode(":",$CTXT . '::');
  if ($t <> '') { 
    $tag .= $mt . ': ' . deficons(trim($t)); 
	} else {
	$tag .= deficons(trim($mt));
  }


return $tag;
} // end gen_tag
// ------------------------------------------------------------------

//  select units to display based on units of measure (UOM)
function load_units($UOM) {

global $dpRain,$dpWind,$dpBaro,$dpTemp,$useKnots,$useMPH,$useMPS,$useMB;

$Units = array();

if ($UOM == 'M') {
    $Units =  array(  // metric with native wind units
    'wind' => 'kph',
	'temp' => '&deg;C',
	'baro' => 'hPa',
	'humid' => '%',
	'rain' => 'mm',
	'dist' => 'km');
	$dpRain = 1;
	$dpBaro = 1;
	$dpTemp = 1;
	$dpWind = 0;
  } else {
    $Units =  array(  // english with native wind units
    'wind' => 'mph',
	'temp' => '&deg;F',
	'baro' => 'in',
	'humid' => '%',
	'rain' => 'in',
	'dist' => 'nm');
	$dpRain = 2;
	$dpBaro = 2;
	$dpTemp = 0;
	$dpWind = 0;

}

 $Units['time'] = '';
 if ($useKnots) { $Units['wind'] = 'kts'; }
 if ($useMPH)   { $Units['wind'] = 'mph'; }
 if ($useMPS)   { $Units['wind'] = 'm/s'; $dpWind = 1; }
 if ($useMB and $UOM == 'M')    { $Units['baro'] = 'mb';}
 
 return $Units;
} // end load_units
// ------------------------------------------------------------------

// generate the rotation JavaScript to browser page
function prt_jscript () {
	global $showTownName;
// NOTE: the following is not PHP, it's JavaScript
//   no changes should be required here.
$t = '<script type="text/javascript">
// <![CDATA[
// ----------------------------------------------------------------------
// Rotate content display -- Ken True -- saratoga-weather.org
//
// --------- begom settomgs ---------------------------------------------------------------
var CMBWNrotatedelay=3000; // Rotate display every 3 secs (= 3000 ms)
// --------- emd settomgs -----------------------------------------------------------------
//
// you shouldn\'t need to change things below this line
//
var ie4=document.all;
var ie8 = false;
if (ie4 && /MSIE (\d+\.\d+);/.test(navigator.userAgent)){ //test for MSIE x.x;
 var ieversion=new Number(RegExp.$1) // capture x.x portion and store as a number
 if (ieversion>=8) {
   ie4=false;
   ie8=true;
 }
}
var CMBWNcurindex = 0;
var CMBWNtotalcontent = 0;
var CMBWNrunrotation = 1;
var browser = navigator.appName;

function get_content_tags ( tag ) {
// search all the span tags and return the list with class=tag 
//
  if (ie4 && browser != "Opera" && ! ie8) {
    var elem = document.getElementsByTagName("span");
	var lookfor = "className";
  } else {
    var elem = document.getElementsByTagName("span");
	var lookfor = "class";
  }
     var arr = new Array();
     for(var i = 0, iarr = 0; i < elem.length; i++) {
          var att = elem[i].getAttribute(lookfor);
          if(att == tag) {
               arr[iarr] = elem[i];
               iarr++;
          }
     }

     return arr;
}


function CMBWN_get_total() {
';
if ($showTownName) {
	$t .= '  CMBWNtotalcontent = 8; // content0 .. content7 ';
} else {
	$t .= '  CMBWNtotalcontent = 7; // content0 .. content6 ';
}
 $t .= '
}

function CMBWN_contract_all() {
  for (var y=0;y<CMBWNtotalcontent;y++) {
      var elements = get_content_tags("CMBWNcontent"+y);
	  var numelements = elements.length;
//	  alert("CMBWN_contract_all: content"+y+" numelements="+numelements);
	  for (var index=0;index!=numelements;index++) {
         var element = elements[index];
		 element.style.display="none";
      }
  }
}

function CMBWN_expand_one(which) {
  CMBWN_contract_all();
  var elements = get_content_tags("CMBWNcontent"+which);
  var numelements = elements.length;
  for (var index=0;index!=numelements;index++) {
     var element = elements[index];
	 element.style.display="inline";
  }
}
function CMBWN_step_content() {
  CMBWN_get_total();
  CMBWN_contract_all();
  CMBWNcurindex=(CMBWNcurindex<CMBWNtotalcontent-1)? CMBWNcurindex+1: 0;
  CMBWN_expand_one(CMBWNcurindex);
}
function CMBWN_set_run(val) {
  CMBWNrunrotation = val;
  CMBWN_rotate_content();
}
function CMBWN_rotate_content() {
  if (CMBWNrunrotation) {
    CMBWN_get_total();
    CMBWN_contract_all();
    CMBWN_expand_one(CMBWNcurindex);
    CMBWNcurindex=(CMBWNcurindex<CMBWNtotalcontent-1)? CMBWNcurindex+1: 0;
    setTimeout("CMBWN_rotate_content()",CMBWNrotatedelay);
  }
}

CMBWN_rotate_content();

// ]]>
</script>
';
$t .= '<script type="text/javascript" src="sorttable.js"></script>
';
return($t);
          
}  // end prt_jscript
// ------------------------------------------------------------------

// initalize the assembly string for CSS
function load_strings () {

  global $CSS,$Graphic,$Version,$Program,$ControlsX, $ControlsY;
  
// top of CSS for mesomap display
  $CSS ='<!-- begin CMBWN-mesomap CSS -->
<!-- generated by ' . $Program . ' ' . $Version . ' -->
<style type="text/css">
#CMBWNmeso {
      background: url(' . $Graphic . ') no-repeat;
      font-family: Tahoma, Arial, Helvetica, sans-serif;
      color: #000088;
      position: relative;
}
#CMBWNmeso p {
      position: absolute;
	  font-family: Tahoma, Arial, Helvetica, sans-serif; 
	  color: #FFFFFF;
	  font-size: 8pt;
	  font-weight: normal;
	  margin: 0 0 0 0;
	  padding: 0 0 0 0;
}
#CMBWNmeso p img {
      border-style: none;
}
#CMBWNmeso img {
      border-style: none;
}
#CMBWNdescr p {
	  font-family: Arial, Helvetica, sans-serif;
	  font-size: 10pt;
	  font-weight: normal;
	  text-align: left;
}
#CMBWNlist {
      color: black; 
	  font-family: Arial, Helvetica, sans-serif; 
	  font-size: 10pt;
}
#CMBWNcontrols {
	  top: ' . $ControlsY . 'px;
	  left: ' . $ControlsX . 'px;
	  font-family: Arial, Helvetica, sans-serif; 
	  font-size: 8pt;
	  font-weight: normal;
	  position: relative;
	  display: inline;
	  padding: 0 0;
	  margin: 0 0;
	  border: none;
	  z-index: 20;
}
.CMBWNtable {
	font-family: Arial, Helvetica, sans-serif;
	font-size: 8pt;
	color: #000000;

}
.CMBWNcontent0 {
	display: inline;
}
.CMBWNcontent1 {
	display: none;
}
.CMBWNcontent2 {
	display: none;
}
.CMBWNcontent3 {
	display: none;
}
.CMBWNcontent4 {
	display: none;
}
.CMBWNcontent5 {
	display: none;
}
.CMBWNcontent6 {
	display: none;
}
.CMBWNcontent7 {
	display: none;
}
.CMBWNcontent8 {
	display: none;
}
'; 

global $htmlstart,$timeFormat,$updatedOn,$htmlnext,$html,$ThisStation,$MapWidth,$MapHeight;

// top boilerplate for include text
  $htmlstart = '<!-- begin included CMBWN text -->
<!-- as of ' . fixtimezonename(strftime($timeFormat,$updatedOn)) . ' -->
<div id="CMBWNmeso"><img src="';  // leave room for toggling the image here

  $htmlnext ='" usemap="#CMBWN" width="'.$MapWidth.'" height="'.$MapHeight.'" 
    alt="Mesomap of Manitoba Weather Network Stations" />
';
  $html = '<div id="CMBWNdescr">'. MESO_BOXPRTA . '  ' . $ThisStation . MESO_BOXPRTB . fixtimezonename(strftime($timeFormat,$updatedOn)) . '
</p></div>
<div id="CMBWNlist"> 
<ul style="list-style: square">
';


} // end load_strings
// ------------------------------------------------------------------

// following code added in 2.10 for conditions and metar decoding


function load_iconDefs () {

global $Icons, $vws_icon, $IconsText;

   // CURRENT CONDITIONS ICONS FOR clientraw.txt
   // create array for icons. There are 35 possible values in clientraw.txt
   // It would be simpler to do this with array() but to make it easier to 
   // modify each element is defined individually. Each index [#] corresponds
   // to the value provided in clientraw.txt
   $Icons[0] = "day_clear.gif";            // imagesunny.visible
   $Icons[1] = "night_clear.gif";          // imageclearnight.visible
   $Icons[2] = "day_partly_cloudy.gif";    // imagecloudy.visible
   $Icons[3] = "day_partly_cloudy.gif";    // imagecloudy2.visible
   $Icons[4] = "night_partly_cloudy.gif";  // imagecloudynight.visible
   $Icons[5] = "day_clear.gif";            // imagedry.visible
   $Icons[6] = "fog.gif";                  // imagefog.visible
   $Icons[7] = "haze.gif";                 // imagehaze.visible
   $Icons[8] = "day_heavy_rain.gif";       // imageheavyrain.visible
   $Icons[9] = "day_mostly_sunny.gif";     // imagemainlyfine.visible
   $Icons[10] = "mist.gif";                // imagemist.visible
   $Icons[11] = "fog.gif";                 // imagenightfog.visible
   $Icons[12] = "night_heavy_rain.gif";    // imagenightheavyrain.visible
   $Icons[13] = "night_cloudy.gif";        // imagenightovercast.visible
   $Icons[14] = "night_rain.gif";          // imagenightrain.visible
   $Icons[15] = "night_light_rain.gif";    // imagenightshowers.visible
   $Icons[16] = "night_snow.gif";          // imagenightsnow.visible
   $Icons[17] = "night_tstorm.gif";        // imagenightthunder.visible
   $Icons[18] = "day_cloudy.gif";          // imageovercast.visible
   $Icons[19] = "day_partly_cloudy.gif";   // imagepartlycloudy.visible
   $Icons[20] = "day_rain.gif";            // imagerain.visible
   $Icons[21] = "day_rain.gif";            // imagerain2.visible
   $Icons[22] = "day_light_rain.gif";      // imageshowers2.visible
   $Icons[23] = "sleet.gif";               // imagesleet.visible
   $Icons[24] = "sleet.gif";               // imagesleetshowers.visible
   $Icons[25] = "day_snow.gif";            // imagesnow.visible
   $Icons[26] = "day_snow.gif";            // imagesnowmelt.visible
   $Icons[27] = "day_snow.gif";            // imagesnowshowers2.visible
   $Icons[28] = "day_clear.gif";           // imagesunny.visible
   $Icons[29] = "day_tstorm.gif";          // imagethundershowers.visible
   $Icons[30] = "day_tstorm.gif";          // imagethundershowers2.visible
   $Icons[31] = "day_tstorm.gif";          // imagethunderstorms.visible
   $Icons[32] = "tornado.gif";             // imagetornado.visible
   $Icons[33] = "windy.gif";               // imagewindy.visible
   $Icons[34] = "day_partly_cloudy.gif";   // stopped rainning
   $Icons[35] = "windyrain.gif";   // wind + rain

   $IconsText[0] =  'Sunny';
   $IconsText[1] =  'Clear';
   $IconsText[2] =  'Cloudy';
   $IconsText[3] =  'Cloudy2';
   $IconsText[4] =  'Partly Cloudy';
   $IconsText[5] =  'Dry';
   $IconsText[6] =  'Fog';
   $IconsText[7] =  'Haze';
   $IconsText[8] =  'Heavy Rain';
   $IconsText[9] =  'Mainly Fine';
   $IconsText[10] = 'Mist';
   $IconsText[11] = 'Fog';
   $IconsText[12] = 'Heavy Rain';
   $IconsText[13] = 'Overcast';
   $IconsText[14] = 'Rain';
   $IconsText[15] = 'Showers';
   $IconsText[16] = 'Snow';
   $IconsText[17] = 'Thunder';
   $IconsText[18] = 'Overcast';
   $IconsText[19] = 'Partly Cloudy';
   $IconsText[20] = 'Rain';
   $IconsText[21] = 'Rain2';
   $IconsText[22] = 'Showers2';
   $IconsText[23] = 'Sleet';
   $IconsText[24] = 'Sleet Showers';
   $IconsText[25] = 'Snow';
   $IconsText[26] = 'Snow Melt';
   $IconsText[27] = 'Snow Showers2';
   $IconsText[28] = 'Sunny';
   $IconsText[29] = 'Thunder Showers';
   $IconsText[30] = 'Thunder Showers2';
   $IconsText[31] = 'Thunder Storms';
   $IconsText[32] = 'Tornado';
   $IconsText[33] = 'Windy';
   $IconsText[34] = 'Stopped Raining';
   $IconsText[35] = 'Wind/Rain';

     // CURRENT CONDITIONS ICONS FOR VWS
     // create array for icons.
     // It would be simpler to do this with array() but to make it easier to 
     // modify each element is defined individually. Each index ["SOME VALUE"] 
     // corresponds to a possible value of the ^climate_cconds1^ tag. Because
     // VWS does not provide a day/night tag, we figure that out immediately 
     // above and then use the $daynight value in the image definitions below
     // to select the right icon.
     $vws_icon['BLIZZARD CONDITION'] = '%s_snow';
     $vws_icon['BLIZZARD'] = '%s_snow';
     $vws_icon['INCREASING CLOUDS'] = '%s_cloudy';
     $vws_icon['BECOMING CLOUDY'] = '%s_cloudy';
     $vws_icon['HAZY'] = 'haze';
     $vws_icon['HAZE'] = 'haze';
     $vws_icon['HZ'] = 'haze';
     $vws_icon['SUN AND CLOUD'] = '%s_partly_cloudy';
     $vws_icon['FEW CLOUD'] = '%s_partly_cloudy';
     $vws_icon['FEW CLOUDS'] = '%s_partly_cloudy';
     $vws_icon['PARTIAL CLEARING'] = '%s_partly_cloudy';
     $vws_icon['CLEARING'] = '%s_partly_cloudy';
     $vws_icon['VARIABLE CLOUDINESS'] = '%s_partly_cloudy';
     $vws_icon['VARIABLE CLOUDS'] = '%s_partly_cloudy';
     $vws_icon['SCATTERED CLOUDS'] = '%s_partly_cloudy';
     $vws_icon['BLOWING SNOW'] = '%s_snow';
     $vws_icon['DRIFTING SNOW'] = '%s_snow';
     $vws_icon['RAIN+SNOW'] = '%s_snow';
     $vws_icon['SNOW+FREEZING RAIN'] = '%s_snow';
     $vws_icon['SNOW+RAIN'] = '%s_snow';
     $vws_icon['RAIN+SNOW+SHOWERS'] = '%s_snow';
     $vws_icon['FREEZING RAIN AND SNOW'] = '%s_snow';
     $vws_icon['RAIN AND SNOW'] = '%s_snow';
     $vws_icon['SNOW AND RAIN'] = '%s_snow';
     $vws_icon['LIGHT SNOW MIST'] = '%s_snow';
     $vws_icon['LIGHT SNOW +MIST'] = '%s_snow';
     $vws_icon['CHANCE OF RAIN OR SNOW'] = '%s_snow';
     $vws_icon['FREEZING RAIN OR SNOW'] = '%s_snow';
     $vws_icon['FREEZING FOG'] = '%s_snow';
     $vws_icon['RAIN OR SNOW'] = '%s_snow';
     $vws_icon['RAIN+MIXED+SNOW'] = '%s_snow';
     $vws_icon['SNOW+MIXED+RAIN'] = '%s_snow';
     $vws_icon['MIXPCPN'] = '%s_snow';
     $vws_icon['SLEET+AND+SNOW'] = '%s_snow';
     $vws_icon['SLEET'] = 'sleet';
     $vws_icon['WINTRY MIX'] = '%s_snow';
     $vws_icon['CHANCE OF SNOW OR RAIN'] = '%s_snow';
     $vws_icon['SNOW OR RAIN'] = '%s_snow';
     $vws_icon['PARTLY|MOSTLY+CLOUDY|SUNNY+THUNDERSTORM'] = '%s_tstorm';
     $vws_icon['CHANCE OF+THUNDERSTORM'] = '%s_tstorm';
     $vws_icon['CHANCE T-STORMS'] = '%s_tstorm';
     $vws_icon['NEARBY THUNDERSTORM'] = '%s_tstorm';
     $vws_icon['LIGHT THUNDERSTORM RAIN'] = '%s_tstorm';
     $vws_icon['HEAVY THUNDERSTORM RAIN'] = '%s_tstorm';
     $vws_icon['THUNDERSTORM'] = '%s_tstorm';
     $vws_icon['T-STORMS'] = '%s_tstorm';
     $vws_icon['TSRA'] = '%s_tstorm';
     $vws_icon['VCTS'] = '%s_tstorm';
     $vws_icon['ISOLATED SHOWER'] = '%s_partly_cloudy';
     $vws_icon['HEAVY SNOW'] = '%s_snow';
     $vws_icon['SNOW SHOWER'] = '%s_snow';
     $vws_icon['PARTLY CLOUDY+SHOWERS LIKELY'] = '%s_partly_cloudy';
     $vws_icon['PARTLY CLOUDY+SHOWER'] = '%s_partly_cloudy';
     $vws_icon['MOSTLY CLOUDY|PARTLY SUNNY+SHOWERS LIKELY'] = '%s_partly_cloudy';
     $vws_icon['MOSTLY CLOUDY|PARTLY SUNNY+SHOWER'] = '%s_partly_cloudy';
     $vws_icon['CHANCE OF SHOWER'] = '%s_rain';
     $vws_icon['SCATTERED SHOWER'] = '%s_rain';
     $vws_icon['RAIN SHOWER'] = '%s_rain';
     $vws_icon['SHOWER'] = '%s_rain';
     $vws_icon['MAINLY CLOUDY'] = '%s_cloudy';
     $vws_icon['CLOUDY PERIODS'] = '%s_partly_cloudy';
     $vws_icon['BROKEN CLOUDS'] = '%s_partly_cloudy';
     $vws_icon['FLURR'] = '%s_snow';
     $vws_icon['FAIR'] = '%s_partly_cloudy';
     $vws_icon['LIGHT SNOW'] = '%s_snow';
     $vws_icon['CHANCE OF SNOW'] = '%s_snow';
     $vws_icon['SNOW'] = '%s_snow';
     $vws_icon['SNOW FOG'] = '%s_snow';
     $vws_icon['SNOW +FOG'] = '%s_snow';
     $vws_icon['SNOW FREEZING FOG'] = '%s_snow';
     $vws_icon['SNOW +FREEZING FOG'] = '%s_snow';
     $vws_icon['HEAVY SNOW'] = '%s_snow';
     $vws_icon['HEAVY SNOW +FOG'] = '%s_snow';
     $vws_icon['HEAVY SNOW FREEZING FOG'] = '%s_snow';
     $vws_icon['FRZDRZL'] = '%s_light_rain';
     $vws_icon['LIGHT DRIZZLE'] = '%s_light_rain';
     $vws_icon['FREEZING DRIZZLE'] = '%s_light_rain';
     $vws_icon['FREEZING RAIN'] = '%s_light_rain';
     $vws_icon['HEAVY RAIN'] = '%s_rain';
     $vws_icon['LIGHT RAIN'] = '%s_light_rain';
     $vws_icon['CHANCE RAIN'] = '%s_light_rain';
     $vws_icon['SHOWERS'] = '%s_light_rain';
     $vws_icon['SHOWERS RAIN'] = '%s_light_rain';
     $vws_icon['LIGHT RAIN+MIST'] = '%s_light_rain';
     $vws_icon['LIGHT RAIN +MIST'] = '%s_light_rain';
     $vws_icon['LIGHT RAIN MIST'] = '%s_light_rain';
     $vws_icon['LIGHT RAIN SHOWERS +MIST'] = '%s_light_rain';
     $vws_icon['LIGHT RAIN SHOWERS'] = '%s_light_rain';
     $vws_icon['RAIN MIST'] = '%s_light_rain';
     $vws_icon['DRIZZLE +MIST'] = '%s_light_rain';
     $vws_icon['LIGHT DRIZZLE MIST'] = '%s_light_rain';
     $vws_icon['PARTLY CLOUDY+CHANCE OF RAIN'] = '%s_partly_cloudy';
     $vws_icon['MOSTLY CLOUDY+CHANCE OF RAIN'] = '%s_partly_cloudy';
     $vws_icon['CHANCE OF RAIN'] = '%s_rain';
     $vws_icon['OCCASIONAL RAIN'] = '%s_rain';
     $vws_icon['HEAVY RAIN +MIST'] = '%s_rain';
     $vws_icon['RAIN'] = '%s_rain';
     $vws_icon['WINDY'] = 'windy';
     $vws_icon['SAND WHIRLS'] = 'windy';
     $vws_icon['BLUSTERY'] = 'windy';
     $vws_icon['NEARBY FOG'] = 'fog';
     $vws_icon['PATCHY FOG'] = 'fog';
     $vws_icon['PATCHES FOG'] = 'fog';
     $vws_icon['PATCHES OF FOG'] = 'fog';
     $vws_icon['PATCHES FOG MIST'] = 'fog';
     $vws_icon['WIDESPREAD FOG'] = 'fog';
     $vws_icon['FOG MIST'] = 'fog';
     $vws_icon['SHALLOW FOG'] = 'fog';
     $vws_icon['FOG+?DRIZZLE'] = '%s_light_rain';
     $vws_icon['PATCHY+DRIZZLE'] = '%s_light_rain';
     $vws_icon['DRIZZLE'] = '%s_light_rain';
	 $vws_icon['LIGHT DRIZZLE +MIST'] = '%s_light_rain';
     $vws_icon['HAZE +MIST'] = 'mist';
     $vws_icon['MIST'] = 'mist';
     $vws_icon['MIST +SHALLOW FOG'] = 'mist';
     $vws_icon['LIGHT RAIN MIST'] = '%s_light_rain';
     $vws_icon['SMOKE'] = 'haze';
     $vws_icon['FROZEN PRECIP'] = '%s_snow';
     $vws_icon['DRY'] = '%s_clear';
     $vws_icon['VARIABLE HIGH CLOUDINESS'] = '%s_partly_cloudy';
     $vws_icon['PARTLY CLOUDY'] = '%s_partly_cloudy';
     $vws_icon['PARTLYCLOUDY'] = '%s_partly_cloudy';
     $vws_icon['MOSTLY CLOUDY'] = '%s_cloudy';
     $vws_icon['MOSTLYCLOUDY'] = '%s_cloudy';
     $vws_icon['MOSTLY SUNNY'] = '%s_mostly_sunny';
     $vws_icon['MOSTLYSUNNY'] = '%s_mostly_sunny';
     $vws_icon['PARTLY SUNNY'] = '%s_partly_cloudy';
     $vws_icon['PARTLYSUNNY'] = '%s_partly_cloudy';
     $vws_icon['SUNNY'] = '%s_clear';
     $vws_icon['INCREASING CLOUDINESS'] = '%s_cloudy';
     $vws_icon['FOG'] = 'fog';
     $vws_icon['CLOUDY'] = '%s_cloudy';
     $vws_icon['OCCASIONAL SUNSHINE'] = '%s_partly_cloudy';
     $vws_icon['PARTIAL SUNSHINE'] = '%s_partly_cloudy';
     $vws_icon['CLOUDS'] = '%s_cloudy';
     $vws_icon['OVERCAST'] = '%s_cloudy';
     $vws_icon['MOSTLY CLEAR'] = '%s_clear';
     $vws_icon['CLEAR'] = '%s_clear';
     $vws_icon['CLEAR SKIES'] = '%s_clear';
     $vws_icon['ICE CRYSTALS'] = '%s_light_rain';
     $vws_icon['NO PRECIPITATION'] = '%s_partly_cloudy';
     $vws_icon['LIGHTNING OBSERVED'] = '%s_tstorm';
     $vws_icon['THUNDER'] = '%s_tstorm';
     $vws_icon['MILD AND BREEZY'] = 'windy';
     $vws_icon['NEARBY BLOWING WIDESPREAD DUST'] = 'dust';
     $vws_icon['WIDESPREAD BLOWING DUST'] = 'dust';
     $vws_icon['BLOWING WIDESPREAD DUST'] = 'dust';
     $vws_icon['WIDESPREAD DUST'] = 'dust';
     $vws_icon['BLOWING WIDESPREAD DUST +HAZE +SQUALLS'] = 'dust';
     $vws_icon['DUST'] = 'dust';
     $vws_icon['HOT AND HUMID'] = '%s_clear';
     $vws_icon['CONTINUED HOT'] = '%s_clear';
     $vws_icon['FILTERED SUNSHINE'] = '%s_cloudy';



return;

}


function decode_VWS_conds ($time,$forecast,$sunrise,$sunset) {
     // Many thanks to Larry at Anole Computer for the basis of
	 // this routine.
	 // adapted by Ken True to be compatible with WD icon set
	 global $vws_icon,$Debug;
	 
	 $Debug .= "<!-- condition begin: '$time','$forecast','$sunrise','$sunset' -->\n";
	 if(!preg_match('/^\d{1,2}:\d{2}[:\d{2}]{0,1}\s*[am|pm]*$/i',$sunrise)) { $sunrise = '';  }
	 if(!preg_match('/^\d{1,2}:\d{2}[:\d{2}]{0,1}\s*[am|pm]*$/i',$sunset)) { $sunset = '';  }
 
     $sunrise2 = fixupTime(($sunrise<>'')?"$sunrise":"6:00a");
     $sunset2 = fixupTime(($sunset<>'')?"$sunset":"7:00p");
     $time2 =   fixupTime(($time<>'')?"$time":date("H:i",time()));
     if ($time2 >= $sunrise2 and $time2 <= $sunset2) {
         $daynight = 'day';
     } // end if
     else {
         $daynight = 'night';
     } // end else
	 $Debug .= "<!-- condition using: time2='$time2' as $daynight for sunrise2='$sunrise2',sunset2='$sunset2'  -->\n";
  $forecast = trim($forecast);
  if ($forecast <> '' and strtoupper($forecast) <> 'N/A') {
     $temp = sprintf($vws_icon[$forecast],$daynight) . '.gif';
	 if ($temp == '.gif') { $temp = ''; }
	 return ($temp);
  } else {
     return ('');
  }
}

function fixupTime ($intime) {
  global $Debug;
  $t = explode(':',$intime);
  if (preg_match('/p/i',$intime)) { $t[0] = $t[0] + 12; }
  if ($t[0] > 23) {$t[0] = 12; }
  if (preg_match('/^12.*a/i',$intime)) { $t[0] = 0; }
  if ($t[0] < '10') {$t[0] = sprintf("%02d",$t[0]); } // leading zero on hour.
  $t2 = join(':',$t); // put time back to gether;
  $t2 = preg_replace('/[^\d\:]/is','',$t2); // strip out the am/pm if any
  $Debug .= "<!-- fixupTime in='$intime' out='$t2' -->\n";
  return($t2);	
	
}

function getBaroTrendArrow($BTRND) {
  global $windArrowDir,$windArrowSize,$showBaroTrendArrow;
  
  $arrows = array(
    'S' => 'W',
	'RS' => 'WSW',
	'R' => 'SW',
	'RR' => 'S',
	'FS' => 'WNW',
	'F'  =>  'NW',
	'FR' =>  'N'
  );
  
  	if ($windArrowSize == 'S') {
	  $windGIF = '-sm.gif';
	  $windSIZE = 'height="9" width="9"';
	} else {
	  $windGIF = '.gif';
	  $windSIZE = 'height="14" width="14"';
	}

  $trend = trim(strtoupper($BTRND));
  $words = explode(" ",$trend . '  ');
  $abbrev= substr($words[0],0,1) . substr($words[1],0,1);
//  $windArrowDir = './arrows/';
  if ($showBaroTrendArrow and $windArrowDir <> '' and isset($arrows[$abbrev])) {
   return("<img src=\"$windArrowDir" . $arrows[$abbrev] . "$windGIF\" $windSIZE 
   alt=\"".defbarotrnd($BTRND)."\" title=\"".defbarotrnd($BTRND)."\" />");
  } else {
   return("");
  }
}
// ---------------------------------------------------------

function process_metar($metar,$icao) {
  global $lang,$Debug, $wxInfo, $metarPtr, $group;
  $Debug .= "<!-- called process_metar -->\n";
	//   This function directs the examination of each group of the METAR. The problem
	// with a METAR is that not all the groups have to be there. Some groups could be
	// missing. Fortunately, the groups must be in a specific order. (This function
	// also assumes that a METAR is well-formed, that is, no typographical mistakes.)
	//   This function uses a function variable to organize the sequence in which to
	// decode each group. Each function checks to see if it can decode the current
	// METAR part. If not, then the group pointer is advanced for the next function
	// to try. If yes, the function decodes that part of the METAR and advances the
	// METAR pointer and group pointer. (If the function can be called again to
	// decode similar information, then the group pointer does not get advanced.)
	$lang = 'en';
  foreach ($wxInfo as $i => $value) { // clear out prior contents
   unset($wxInfo[$i]);
//    $wxInfo[$i] = '';
  }
//  $Debug .= "<!-- wxInfo cleared\n" . print_r($wxInfo,true) . " -->\n";
  $wxInfo['STATION'] = $icao;
	
	if ($metar != '') {
		$metarParts = explode(' ',$metar);
		$groupName = array('get_station',
                       'get_time',
                       'get_station_type',
                       'get_wind',
                       'get_var_wind',
                       'get_visibility',
                       'get_runway',
                       'get_conditions',
                       'get_cloud_cover',
                       'get_temperature',
                       'get_altimeter');
		$metarPtr = 3;  // get_station identity is ignored
		$group = 1; // start with Time
		
		while ($group < count($groupName)) {
			$part = $metarParts[$metarPtr];
			$Debug .= "<!-- calling '" . $groupName[$group] . "' part='$part' ptr=$metarPtr grp=$group -->\n";
			$groupName[$group]($part);  // $groupName is a function variable
			}
	} else {
	  $wxInfo['ERROR'] = 'Data not available';
	}
}

//----------------------------------------------------------------
// Ignore station code. Script assumes this matches requesting
// $station. This function is never called. It is here for
// completeness of documentation.
function get_station($part)
{ global $lang,$Debug, $wxInfo, $metarPtr, $group;
  if (strlen($part) == 4 and $group == 0)
  {
    $group++;
    $metarPtr++;
  }
}

function get_time($part)
{ global $lang,$Debug, $wxInfo, $metarPtr, $group;
  // Ignore observation time. This information is found in the
  // first line of the NWS file.
  // Format is ddhhmmZ where dd = day, hh = hours, mm = minutes
  // in UTC time.
  if (substr($part,-1) == 'Z') {
     $dd = substr($part,0,2);
	 $hh = substr($part,2,2);
	 $mm = substr($part,4,2);
	 
     $metarPtr++;
  }
  $group++;
}

function get_station_type($part)
{ global $lang,$Debug, $wxInfo, $metarPtr, $group;
  // Ignore station type if present.
  if ($part == 'AUTO' || $part == 'COR')
    $metarPtr++;
  $group++;
}

  function speed($part, $unit)
  {
    global $lang,$Debug, $wxInfo, $metarPtr, $group;
    // Convert wind speed into miles per hour.
    // Some other common conversion factors (to 6 significant digits):
    //   1 mi/hr = 1.15080 knots  = 0.621371 km/hr = 2.23694 m/s
    //   1 ft/s  = 1.68781 knots  = 0.911344 km/hr = 3.28084 m/s
    //   1 knot  = 0.539957 km/hr = 1.94384 m/s
    //   1 km/hr = 1.852 knots  = 3.6 m/s
    //   1 m/s   = 0.514444 knots = 0.277778 km/s
    if ($unit == 'KT') 
      $speed = 1.1508 * $part;    // from knots
    elseif ($unit == 'MPS')
      $speed = 2.23694 * $part;   // from meters per second
    else
      $speed = 0.621371 * $part;  // from km per hour
    $speedkph = $speed / 0.621371;
    if ($lang=="en")
      $speed ="" . round($speed) . " mph (". round($speedkph) . " km/h)";
    else
      $speed = "" . round($speedkph) . " km/h";
    return $speed;
  }

//-------------------------------------------------------------------------
// Decodes wind direction and speed information.
// Format is dddssKT where ddd = degrees from North, ss = speed,
// KT for knots  or dddssGggKT where G stands for gust and gg = gust
// speed. (ss or gg can be a 3-digit number.)
// KT can be replaced with MPH for meters per second or KMH for
//kilometers per hour.
function get_wind($part)
{ global $lang,$Debug, $wxInfo, $metarPtr, $group;
  
  
  if (preg_match('/^([0-9G]{5,10}|VRB[0-9]{2,3})(KT|MPS|KMH)$/',$part,$pieces))
  {
    $part = $pieces[1];
    $unit = $pieces[2];
    if ($part == '00000')
    {
      $wxInfo['WIND'] = 'calm';  // no wind
    }
    else
    {
      preg_match('/([0-9]{3}|VRB)([0-9]{2,3})G?([0-9]{2,3})?/',$part,$pieces);
      if ($pieces[1] == 'VRB')
        $direction = 'varies';
      else
      {
        $angle = (integer) $pieces[1];
        $compass = array('N','NNE','NE','ENE','E','ESE','SE','SSE',
                         'S','SSW','SW','WSW','W','WNW','NW','NNW');
        $direction = $compass[round($angle / 22.5) % 16];
      }
      if (!isset($pieces[3]) or (isset($pieces[3]) and $pieces[3] == 0 ) ) {
        $gust = '';
	   } else {
        $gust = ', gusting to ' . speed($pieces[3], $unit);
	   }
      $wxInfo['WIND'] = $direction . ' at ' . speed($pieces[2], $unit) . $gust;
    }
    $metarPtr++;
  }
  $group++;
}

function get_var_wind($part)
{ global $lang,$Debug, $wxInfo, $metarPtr, $group;
  // Ignore variable wind direction information if present.
  // Format is fffVttt where V stands for varies from fff
  // degrees to ttt degrees.
  if (preg_match('/([0-9]{3})V([0-9]{3})/',$part,$pieces))
    $metarPtr++;
  $group++;
}


//------------------------------------------------------------------
// Decodes visibility information. This function will be called a
// second time if visibility is limited to an integer mile plus a
// fraction part.
// Format is mmSM for mm = statute miles, or m n/dSM for m = mile
// and n/d = fraction of a mile, or just a 4-digit number nnnn (with
// leading zeros) for nnnn = meters.

function get_visibility($part) {

  global $lang,$Debug, $wxInfo, $metarPtr, $group;
  static $integerMile = '';
  if (strlen($part) == 1)
  {
    // visibility is limited to a whole mile plus a fraction part
    $integerMile = $part . ' ';
    $metarPtr++;
  }
  elseif (substr($part,-2) == 'SM')
  {
    // visibility is in miles
    $part = substr($part,0,strlen($part)-2);
    if (substr($part,0,1) == 'M')
    {
      $prefix = 'less than ';
      $part = substr($part, 1);
    }
    else
      $prefix = '';

    if ($lang == "en")
    {
      if (($integerMile == '' && preg_match('![/]!',$part,$pieces)) || $part == '1')
        $unit = ' mile';
      else
        $unit = ' miles';
    }
    $kmVis = round( $part * 1.6 );
    if ($lang=="en")
      $wxInfo['VISIBILITY'] = $prefix . $integerMile . 
                              " $part $unit ($kmVis km)";
    else
      $wxInfo['VISIBILITY'] = "$kmVis km";
    $metarPtr++;
    $group++;
  }
  elseif (substr($part,-2) == 'KM')
  {
    // unknown (Reported by NFFN in Fiji)
    $metarPtr++;
    $group++;
  }
  elseif (preg_match('/^([0-9]{4})$/',$part,$pieces))
  {
    // visibility is in meters
    $distance = round($part/ 621.4, 1);      // convert to miles
    if ($distance > 5)
      $distance = round($distance);
    if ($distance <= 1)
      $unit = ' mile';
    else
      $unit = ' miles';
    $wxInfo['VISIBILITY'] = $distance . $unit;
    $metarPtr++;
    $group++;
  }
  elseif ($part == 'CAVOK')
  {
    // good weather
    $wxInfo['VISIBILITY'] = 'greater than 7 miles';  // or 10 km
    $wxInfo['CONDITIONS'] = 'Clear';
    $wxInfo['CLOUDS'] = 'clear skies';
    $metarPtr++;
    $group += 4;  // can skip the next 3 groups
  }
  else
  {
    $group++;
  }
}

function get_runway($part)
{ global $lang,$Debug, $wxInfo, $metarPtr, $group;
  // Ignore runway information if present. Maybe called a second time.
  // Format is Rrrr/vvvvFT where rrr = runway number and
  // vvvv = visibility in feet.
  if (substr($part,0,1) == 'R')
    $metarPtr++;
  else
    $group++;
}



function get_conditions($part) {
global $lang,$Debug, $wxInfo, $metarPtr, $group;
	// Decodes current weather conditions. This function maybe called several times
	// to decode all conditions. To learn more about weather condition codes, visit section
	// 12.6.8 - Present Weather Group of the Federal Meteorological Handbook No. 1 at
	// www.nws.noaa.gov/oso/oso1/oso12/fmh1/fmh1ch12.htm
	static $conditions = '';
        $Debug .= "<!-- conditions='$conditions' on entry -->\n";
	static $wxCode = array(
		'VC' => 'Nearby',
		'MI' => 'Shallow',
		'PR' => 'Partial',
		'BC' => 'Patches of',
		'DR' => 'Low Drifting',
		'BL' => 'Blowing',
		'SH' => 'Showers',
		'TS' => 'Thunderstorm',
		'FZ' => 'Freezing',
		'DZ' => 'Drizzle',
		'RA' => 'Rain',
		'SN' => 'Snow',
		'SG' => 'Snow Grains',
		'IC' => 'Ice Crystals',
		'PE' => 'Ice Pellets',
		'GR' => 'Hail',
		'GS' => 'Small Hail',  // and/or snow pellets
		'UP' => 'Unknown',
		'BR' => 'Mist',
		'FG' => 'Fog',
		'FU' => 'Smoke',
		'VA' => 'Volcanic Ash',
		'DU' => 'Widespread Dust',
		'SA' => 'Sand',
		'HZ' => 'Haze',
		'PY' => 'Spray',
		'PO' => 'Well-developed Dust/Sand Whirls',
		'SQ' => 'Squalls',
		'FC' => 'Funnel Cloud, Tornado, or Waterspout',
		'SS' => 'Sandstorm/Duststorm');
	if (preg_match('/^(-|\+|VC)?(TS|SH|FZ|BL|DR|MI|BC|PR|RA|DZ|SN|SG|GR|GS|PE|IC|UP|BR|FG|FU|VA|DU|SA|HZ|PY|PO|SQ|FC|SS|DS)+$/',$part,$pieces)) {
	    $Debug .= "<!-- get_conditions part='$part' -->\n";
		if (strlen($conditions) == 0) $join = '';
		else $join = '+';
		if (substr($part,0,1) == '-') {
			$prefix = 'light ';
			$part = substr($part,1);
			}
		elseif (substr($part,0,1) == '+') {
			$prefix = 'heavy ';
			$part = substr($part,1);
			}
		else $prefix = '';  // moderate conditions have no descriptor
		$conditions .= $join . $prefix;
		// The 'showers' code 'SH' is moved behind the next 2-letter code to make the English translation read better.
		if (substr($part,0,2) == 'SH') $part = substr($part,2,2) . substr($part,0,2). substr($part, 4);
		while ($code = substr($part,0,2)) {
			$conditions .= $wxCode[$code] . ' ';
			$part = substr($part,2);
			}
		$wxInfo['CONDITIONS'] = $conditions;
//        $Debug .= "<!-- conditions='$conditions' metarPtr incr -->\n";
		$metarPtr++;
		}
	else {
		$wxInfo['CONDITIONS'] = $conditions;
		$group++;
//        $Debug .= "<!-- conditions='$conditions' group incr -->\n";
		$conditions = '';
//        $Debug .= "<!-- conditions='$conditions' reset -->\n";
		}
}

function get_cloud_cover($part) {
global $lang,$Debug, $wxInfo, $metarPtr, $group;
	// Decodes cloud cover information. This function maybe called several times
	// to decode all cloud layer observations. Only the last layer is saved.
	// Format is SKC or CLR for clear skies, or cccnnn where ccc = 3-letter code and
	// nnn = altitude of cloud layer in hundreds of feet. 'VV' seems to be used for
	// very low cloud layers. (Other conversion factor: 1 m = 3.28084 ft)
	static $cloudCode = array(
		'SKC' => 'Clear',
		'CLR' => 'Clear',
		'FEW' => 'Partly Cloudy',
		'FW'  => 'Partly Cloudy',
		'SCT' => 'Scattered Clouds',
		'BKN' => 'Mostly Cloudy',
		'OVC' => 'Overcast',
		'VV'  => 'vertical visibility');
	$Debug .= "<!-- get cloud cover '$part' -->\n";
	if ($part == 'SKC' || $part == 'CLR') {
		$wxInfo['CLOUDS'] = $cloudCode[$part];
		$metarPtr++;
		$group++;
		}
	else {
		if (preg_match('/^([A-Z]{2,3})([0-9]{3})/',$part,$pieces)) {  // codes for CB and TCU are ignored
			$wxInfo['CLOUDS'] = $cloudCode[$pieces[1]];
			if ($pieces[1] == 'VV') {
				$altitude = (integer) 100 * $pieces[2];  // units are feet
//				$wxInfo['CLOUDS'] .= " to $altitude ft";
				$wxInfo['CLOUDS'] = "Overcast";
				}
			$metarPtr++;
			}
		else {
			$group++;
			}
		}
}

  function get_heat_index($tempF, $rh)
  { global $lang,$Debug, $wxInfo, $metarPtr, $group;
    // Calculate Heat Index based on temperature in F and relative
    //humidity (65 = 65%)
    if ($tempF > 79 && $rh > 39)
    {
      $hiF = -42.379 + 2.04901523 * $tempF + 10.14333127 * 
             $rh - 0.22475541 * $tempF * $rh;
      $hiF += -0.00683783 * pow($tempF, 2) - 0.05481717 * pow($rh, 2);
      $hiF += 0.00122874 * pow($tempF, 2) * $rh + 0.00085282 * $tempF 
             * pow($rh, 2);
      $hiF += -0.00000199 * pow($tempF, 2) * pow($rh, 2);
      $hiF = round($hiF);
      $hiC = round(($hiF - 32) / 1.8);
      $wxInfo['HEAT INDEX'] = "$hiF&deg;F ($hiC&deg;C)";
    }
  }

  function get_wind_chill($tempF)
  {
    global $lang,$Debug, $wxInfo, $metarPtr, $group;

    // Calculate Wind Chill Temperature based on temperature in F and
    // wind speed in miles per hour
    if ($tempF < 51 && $wxInfo['WIND'] != 'calm')
    {
      $pieces = explode(' ', $wxInfo['WIND']);
      $windspeed = (integer) $pieces[2]; // wind speed must be in mph
      if ($windspeed > 3)
      {
        $chillF = 35.74 + 0.6215 * $tempF - 35.75 * pow($windspeed, 0.16) + 
                  0.4275 * $tempF * pow($windspeed, 0.16);
        $chillF = round($chillF);
        $chillC = round(($chillF - 32) / 1.8);
        $wxInfo['WIND CHILL'] = "$chillF&deg;F ($chillC&deg;C)";
      }
    }
  }

//-------------------------------------------------------------------------
// Decodes temperature and dew point information. Relative humidity is
// calculated. Also, depending on the temperature, Heat Index or Wind
// Chill Temperature is calculated.
// Format is tt/dd where tt = temperature and dd = dew point temperature.
// All units are in Celsius. A 'M' preceeding the tt or dd indicates a
// negative temperature. Some stations do not report dew point, so the
// format is tt/ or tt/XX.
function get_temperature($part)
{
  global $lang, $Debug, $wxInfo, $metarPtr, $group;

  if (preg_match('/^(M?[0-9]{2})\/(M?[0-9]{2}|[X]{2})?$/',$part,$pieces))
  {
    $tempC = (integer) strtr($pieces[1], 'M', '-');
    $tempF = round(1.8 * $tempC + 32);
    if ($lang=="en")
        $wxInfo['TEMP'] = $tempF . "F (" . $tempC . "C)";
    else
        $wxInfo['TEMP'] = $tempC . "C";
    get_wind_chill($tempF);
    if (strlen($pieces[2]) != 0 && $pieces[2] != 'XX')
    {
      $dewC = (integer) strtr($pieces[2], 'M', '-');
      $dewF = round(1.8 * $dewC + 32);
      if ($lang == "en")
          $wxInfo['DEWPT'] = $dewF . "F (" . $dewC . "C)";
      else
          $wxInfo['DEWPT'] = $dewC . "C";
      $rh = round(100 * pow((112 - (0.1 * $tempC) + $dewC) / 
                                                (112 + (0.9 * $tempC)), 8));
      $wxInfo['HUMIDITY'] = $rh . '%';
      get_heat_index($tempF, $rh);
    }
    $metarPtr++;
    $group++;
  }
  else
  {
    $group++;
  }
}


//-----------------------------------------------------------------------
// Decodes altimeter or barometer information.
// Format is Annnn where nnnn represents a real number as nn.nn in
// inches of Hg,
// or Qpppp where pppp = hectoPascals.
// Some other common conversion factors:
//   1 millibar = 1 hPa
//   1 in Hg = 0.02953 hPa
//   1 mm Hg = 25.4 in Hg = 0.750062 hPa
//   1 lb/sq in = 0.491154 in Hg = 0.014504 hPa
//   1 atm = 0.33421 in Hg = 0.0009869 hPa

function get_altimeter($part)
{  global $Debug, $wxInfo, $metarPtr, $group;
  if (preg_match('/^(A|Q)([0-9]{4})/',$part,$pieces))
  {
    if ($pieces[1] == 'A')
    {
      $pressureIN = substr($pieces[2],0,2) . '.' . substr($pieces[2],2);
      // units are inches Hg, converts to hectoPascals
      $pressureHPA = round($pressureIN / 0.02953);
    }
    else
    {
      $pressureHPA = (integer) $pieces[2];        // units are hectoPascals
      $pressureIN = round(0.02953 * $pressureHPA,2);  // convert to inches Hg
    }
    $wxInfo['BAROMETER'] = "$pressureHPA hPa ($pressureIN in Hg)";
    $metarPtr++;
    $group++;
  }
  else
  {
    $group++;
  }
}

// end of the process_metar function set
//----------------------------------------------


?>